/*  bc635.c - Datum BC635/7 PCI VxWorks Driver */

/*
  *******************************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/


#ifndef __INCBC635c
#define __INCBC635c

#ifdef __cplusplus
extern  "C"
{
#endif

/*
modification history
--------------------
12jun00,spk: index arg in tfpDevInstall now used to build device name
             (ex, index = 1 ===> devName = "/tfp1")

9/21/00 pdf  modified for pSOS+ version 2.5

*/

/*
Description

This file contains

*/

  /* #include "sys_conf.h" */
#include "psos.h"
#include <stdio.h>
#include <time.h>
#include "bsp.h"
#include <bspfuncs.h>
#include "src/raven.h"
#include "powerpc/isrppc.h"
#include "../devices/pci/pcihdr.h"
#include "generalcore.h"
#include "integer_type.h"
#include "bc635_mnr.h"
#include "bc635P_mnr.h"

/* defines */
#define EVENT_TIMEOUT       200				/* 2 second timeout */
#define GPS_TIMEOUT         300             /* 3 second timeout */
#define BC625_INT_PRIORITY  0x7				/* MPIC Interrupt priority */

#ifndef MAX
#define MAX(A, B)         ((A) > (B) ? (A) : (B))
#endif

#ifndef MIN
#define MIN(A, B)         ((A) < (B) ? (A) : (B))
#endif


#define bc635PrintDebug \
     if (bc635Debug) printf


/* local declarations */
TFP_DEV *pTfpDev;


/* imports */

  int bc635Debug = 0;
  IMPORT int bc635IntsDebug;

/* global declarations */
  int     TfpDrvNum;

/* forward declarations */
  VOID    tfpDrvIsr (TFP_DEV * pDev);


/******************************************************************************
*
* tfpInit - TFP Initialization function
*
* This function performs all the BC635 initialization and must be the
* first function called.  It assumes that PciConfig() has been called
* by the root task and it calls tfpDevInstall().
*
* RETURNS: OK or ERROR
*/

  /* STATUS tfpInit(TFP_DEV **pTfp) */
STATUS tfpInit()
{
    int     i, index, intvec;
    int     pciBridgeNo;
    int     pciBusNo;
    int     pciDevNo;
    int     pciFuncNo;

    CompactPciInfo cpci_info;
    TFP_DEV *pTfpDev1;  /* pointer to malloc'd structure that contains PCI
			   device info */
    PCI_LOC *p_loc;

    UINT32  addr;
    UINT32  vecAddr;
    UINT32  rVal;

    char    model[80];
    char    devName[80];


	index = 0;								/* Look for first BC635 */

    /* get bus/device/function numbers */
    if (pciFindDevice (BC635_PCI_VENDOR_ID, BC635_PCI_DEVICE_ID,
                       index, &pciBridgeNo, &pciBusNo, &pciDevNo,
                       &pciFuncNo, &p_loc) == ERROR)
      {
        bc635PrintDebug("BC635/7(TFP): device not installed\n");
        return (ERROR);
      }

	/* Set up Interrupt Vector on remote bus */
/*  	if (pciBusNo != 0) */
/*  	  PciSubBusSetIrq(p_loc->hostBridge, p_loc, &cpci_info);  */

    /* device exists - allocate driver dev header */
    if ((pTfpDev1 = (TFP_DEV *)malloc (sizeof (TFP_DEV))) == NULL)
      {
        bc635PrintDebug("BC635/7(TFP): malloc failed\n");
        return (ERROR);
      }

    /* Set return pointer */
    pTfpDev = pTfpDev1;  

    /* Clear some fields */
    pTfpDev1->eventmutex = NULL;
    pTfpDev1->timemutex = NULL;
    pTfpDev1->cmdmutex = NULL;

    pTfpDev1->inthook = (VOIDFUNCPTR) NULL;
    pTfpDev1->expectedInts = 0;
	pTfpDev1->timeout = 0;

	/* Set pSOS PCI location */
	pTfpDev1->p_loc = p_loc;

    /* Get Interrupt Vector */
	intvec = p_loc->intrVec;

    /* Get register base addr (BAR0) */
    pciConfigInLong (pciBridgeNo, pciBusNo, pciDevNo, pciFuncNo,
                     PCI_CFG_BASE_ADDRESS_0, &addr);

    /* Convert pci bus based address to address as seen from CPU */
    pTfpDev1->baseAdrsRegs = PciBase2CpuAddr(addr, pciBridgeNo);
    bc635PrintDebug("tfp: regs base addrs (as seen from CPU): 0x%08X\n",
           pTfpDev1->baseAdrsRegs);

    /* Get DPRAM base addr (BAR1) */
    pciConfigInLong (pciBridgeNo, pciBusNo, pciDevNo, pciFuncNo,
                     PCI_CFG_BASE_ADDRESS_1, &addr);

    /* Convert pci bus based address to address as seen from CPU */
    pTfpDev1->baseAdrsDpram = PciBase2CpuAddr(addr, pciBridgeNo);
    bc635PrintDebug("tfp: dpram base addrs (as seen from CPU): 0x%08X\n",
           pTfpDev1->baseAdrsDpram);

    /* Determine address of tfp command 'input' area */
    pTfpDev1->baseAdrsInput = pTfpDev1->baseAdrsDpram +
      (TFP_IN_BYTE (TFP_DPR_IN_ADRS_MSB (pTfpDev1->baseAdrsDpram)) << 8) +
      TFP_IN_BYTE (TFP_DPR_IN_ADRS_LSB (pTfpDev1->baseAdrsDpram));
    bc635PrintDebug("tfp: dpram input base addrs: 0x%08X\n",
           pTfpDev1->baseAdrsInput);

    /* Determine address of tfp command 'output' area */
    pTfpDev1->baseAdrsOutput = pTfpDev1->baseAdrsDpram +
      (TFP_IN_BYTE (TFP_DPR_OUT_ADRS_MSB (pTfpDev1->baseAdrsDpram)) << 8) +
      TFP_IN_BYTE (TFP_DPR_OUT_ADRS_LSB (pTfpDev1->baseAdrsDpram));
    bc635PrintDebug("tfp: dpram output base addrs: 0x%08X\n",
           pTfpDev1->baseAdrsOutput);

    /* Determine address of tfp 'gps packet' area */
    pTfpDev1->baseAdrsGpsPkt = pTfpDev1->baseAdrsDpram +
      (TFP_IN_BYTE (TFP_DPR_GPS_ADRS_MSB (pTfpDev1->baseAdrsDpram)) << 8) +
      TFP_IN_BYTE (TFP_DPR_GPS_ADRS_LSB (pTfpDev1->baseAdrsDpram));
    bc635PrintDebug("tfp: dpram gps packet base addrs: 0x%08X\n",
           pTfpDev1->baseAdrsGpsPkt);

    /* Determine address of tfp 'year' area */
    pTfpDev1->baseAdrsYr = pTfpDev1->baseAdrsDpram +
      (TFP_IN_BYTE (TFP_DPR_YR_ADRS_MSB (pTfpDev1->baseAdrsDpram)) << 8) +
      TFP_IN_BYTE (TFP_DPR_YR_ADRS_LSB (pTfpDev1->baseAdrsDpram));
    bc635PrintDebug("tfp: dpram year base addrs: 0x%08X\n",
           pTfpDev1->baseAdrsYr);

    /*
     * Verify functionality of tfp command interface by executing 'get
     * model' cmd.
     */
    if (tfpCmdGetTfpModel (pTfpDev1, model) == ERROR)
      {
        bc635PrintDebug("BC635/7(TFP): command interface failed\n");
        tfpDrvFree (pTfpDev1);
        return (ERROR);
      }

    /*
     * Model string should be: "BC635PCI" or "BC637PCU". Check first 4
     * letters since these are always the same.
     */
    pTfpDev1->gps_supported = FALSE;
    if (strncmp ("BC635", model, 5) != 0)
      if (strncmp ("BC637", model, 5) == 0)
        {
          pTfpDev1->gps_supported = TRUE;
          bc635PrintDebug("BC635/7(TFP): BC637 Detected - GPS Facilities Enabled\n");
        }
      else
        {
          bc635PrintDebug("BC635/7(TFP): command test failed\n");
          tfpDrvFree (pTfpDev1);
          return (ERROR);
        }

    /* Create Mutexes */
    if ((rVal = mu_create("BCev", MU_LOCAL, 0,
						  &(pTfpDev1->eventmutex))) != NULL)
      {
        bc635PrintDebug("BC635/7(TFP): Can't Allocate Mutex: 0x%X\n", rVal);
        tfpDrvFree (pTfpDev1);
        return (ERROR);
      }

    if ((rVal = mu_create("BCti", MU_LOCAL, 0,
						  &(pTfpDev1->timemutex))) != NULL)
      {
        bc635PrintDebug("BC635/7(TFP): Can't Allocate Mutex: 0x%X\n", rVal);
        tfpDrvFree (pTfpDev1);
        return (ERROR);
      }

    if ((rVal = mu_create("BCcm", MU_LOCAL, 0,
						  &(pTfpDev1->cmdmutex))) != NULL)
      {
        bc635PrintDebug("BC635/7(TFP): Can't Allocate Mutex: 0x%X\n", rVal);
        tfpDrvFree (pTfpDev1);
        return (ERROR);
      }

    /* Get Board Interrupt Vector Address */
	vecAddr = getMpicVecOffset(intvec);
	bc635PrintDebug("BC635/7(TFP): Interrupt Vector: 0x%X  "
					"Address: 0x%X\n", intvec, vecAddr);

	/* Attach ISR to vector */
    if (PssSetIntHandler(intvec, &tfpDrvIsr, (void *)(pTfpDev1), 1) < 0)
      return(ERROR);

    pTfpDev1->intVector = intvec;     /* Store Vector */

    return (OK);
  }


  /*   void tfpDrvFree (TFP_DEV * pTfpDev) */
void tfpDrvFree ()
  {
    if (pTfpDev->eventmutex != NULL)
      mu_delete(pTfpDev->eventmutex);

    if (pTfpDev->timemutex != NULL)
      mu_delete(pTfpDev->timemutex);

    if (pTfpDev->cmdmutex != NULL)
      mu_delete(pTfpDev->cmdmutex);

    (void) free (pTfpDev);
  }


/*******************************************************************************
* tfpDrvIsr - BC635/7 Driver Interrupt Service Routine
*
*       This ISR provides for mutual exclusion for multiple tasks
*   using this module.  pDev is associated with the task that is
*   expecting an interrupt.  ev_send is associated with the same
*   task and sends only those events that are active and have been
*   requested.  Any other events used by another task are save.
*
*/
  VOID tfpDrvIsr(TFP_DEV * pDev)
  {
        UINT32  intstat, intmask, eventmask;

        /* make sure we got some interrupt */
        TFP_IN_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), &intstat);
        TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);

        /* check for interrupts */
        if ((intstat = intstat & INT_BITMASK) == 0)
            return;

        if ((pDev->expectedInts & INT_BITMASK) == 0)
        { /* Not expecting any interrupts, just clear and return */
            TFP_OUT_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), intstat);
            return;
        }

        eventmask = 0;                      /* Clear return events */

        if (intstat & BCINT_EVENT)
        {                                   /* Process EVENT Interrupt */
            if (pDev->expectedInts & BCINT_EVENT)
                pDev->expectedInts = pDev->expectedInts & ~BCINT_EVENT;
                intmask = intmask & ~BCINT_EVENT;
                eventmask = eventmask | BCINT_EVENT;
        }

        if (intstat & INT_PERIODIC)
        {                                   /* Process PERIODIC Interrupt */
            if (pDev->expectedInts & INT_PERIODIC)
                pDev->expectedInts = pDev->expectedInts & ~INT_PERIODIC;
                intmask = intmask & ~INT_PERIODIC;
                eventmask = eventmask | INT_PERIODIC;
        }

        if (intstat & INT_STROBE)
        {                                   /* Process STROBE Interrupt */
            if (pDev->expectedInts & INT_STROBE)
                pDev->expectedInts = pDev->expectedInts & ~INT_STROBE;
                intmask = intmask & ~INT_STROBE;
                eventmask = eventmask | INT_STROBE;
        }

        if (intstat & INT_1PPS)
        {                                   /* Process 1PPS Interrupt */
            if (pDev->expectedInts & INT_1PPS)
                pDev->expectedInts = pDev->expectedInts & ~INT_1PPS;
                intmask = intmask & ~INT_1PPS;
                eventmask = eventmask | INT_1PPS;
        }

        if (intstat & INT_PACKET)
        {                                   /* Process GPS Packet Interrupt */
            if (pDev->expectedInts & INT_PACKET)
                pDev->expectedInts = pDev->expectedInts & ~INT_PACKET;
                intmask = intmask & ~INT_PACKET;
                eventmask = eventmask | INT_PACKET;
        }

/* Reset the interrupt mask, clear status, and send events */
        TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
        TFP_OUT_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), intstat);
        ev_send(pDev->taskID, intstat);
  }


/*******************************************************************************
* tfpOpen - Open an instance of the driver
*
*/
  TFP_DEV *tfpOpen (TFP_DEV *pDev)
  {

        bc635PrintDebug("tfpOpen: opened device\n");
        return (pDev);

  }


/*******************************************************************************
* tfpClose - close device
*
*/
  STATUS  tfpClose(TFP_DEV *pDev)
  {
        bc635PrintDebug("tfpClose: closed device\n");
        return (OK);
  }


/****************************************************************************
* tfpIoctl - perform I/O control on TFP device
*
*/
  /* STATUS  tfpIoctl (TFP_DEV *pDev, int ioReqCode, int arg) */
  STATUS  tfpIoctl (int ioReqCode, int arg)
  {
      UINT32  tmMinor;
      UINT32  tmMajor;
      TFP_TV *pTv;
      TFP_TM *pTm;

      bc635PrintDebug("tfpIoctl: ioReqCode=%d, arg=x%08X\n", ioReqCode, arg);

	  t_ident((char *)NULL, 0, &(pTfpDev->taskID)); /* Get ev_send Task ID */

      switch  (ioReqCode)
        {
        case SENDGPSPACKET:

/*---------------------------------------------------------------------*/
/* This command sends a GPS data packet to a connected GPS receiver.   */
/* It is applicable only to the BC637.                                 */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
                /* stuff cmd arg bytes in tfp input area */
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SENDGPSPACKET);
          {
            int     i;
            TFP_GPS_PACKET *packet = (TFP_GPS_PACKET *)arg;

            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, packet->packet_length);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, packet->packet_id);

            for (i = 0; i < packet->packet_length - 1; i++)
              TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3 + i,
                            packet->packet_data[i]);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug(" SENDGPSPACKET: timeout waiting "
								"for cmd interface\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }
          }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case REQGPSPACKET:

/*---------------------------------------------------------------------*/
/* This command request a GPS data packet (i.e. position, velocity,    */
/* status, etc.) from a connected GPS receiver.                        */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          /* Clear bit 2 of ack register */
          TFP_OUT_LONG (TFP_REG_ACK (pTfpDev->baseAdrsRegs), TFP_ACK_GPS_CLR);
          /* stuff cmd arg bytes in tfp input area */
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_REQGPSPACKET);
          {
            int     i;
			UINT32  intmask, retval, event;

            TFP_GPS_PACKET *packet = (TFP_GPS_PACKET *) arg;

            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, packet->packet_id);

            /* Get ready for GPS Interrupt */
			intmask = splx(MAX_ILEV);
			pTfpDev->expectedInts = pTfpDev->expectedInts | INT_PACKET;
			splx(intmask);

			TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
			intmask = intmask | INT_PACKET;
			TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            /* Send the Command */
            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug(" SENDGPSPACKET: timeout waiting for "
								"cmd interface\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            /* Interrupt might have already happened, but that's ok */
            retval = ev_receive(INT_PACKET, EV_WAIT | EV_ANY,
                                GPS_TIMEOUT, &event); /* Wait up to 3 sec */
            if (retval)
              packet->packet_length = 0;
            else
              {
                packet->packet_length = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt);
                if (packet->packet_length > 0)
                  packet->packet_id = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt + 1);

                for (i = 0; i < packet->packet_length - 1; i++)
                  {
                    packet->packet_data[i] = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt + 2 + i);
                  }
              }
          }
          mu_unlock(pTfpDev->cmdmutex);
          break;

        case MANREQGPSPACKET:

/*---------------------------------------------------------------------*/
/* Manually Requested GPS Receiver Command                             */
/* This command is a hybrid of the REQGPSPACKET (0x31) & SENDGPSPACKET */
/* commands.  The user specifies the packet length and ID of a packet  */
/* sent by the Acutime and also specifies the packet length, ID, and   */
/* data for the packet to be sent to the Acutime.                      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          /* Clear bit 2 of ack register */
          TFP_OUT_LONG (TFP_REG_ACK (pTfpDev->baseAdrsRegs), TFP_ACK_GPS_CLR);
          /* stuff cmd arg bytes in tfp input area */
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_MANREQGPSPACKET);
          {
            int     i;
			UINT32  intmask, retval, event;

            TFP_GPS_SENDRECEIVE_PACKET *packet = (TFP_GPS_SENDRECEIVE_PACKET *)arg;
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, packet->send.packet_length);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, packet->send.packet_id);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, packet->receive.packet_length);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 4, packet->receive.packet_id);

            for (i = 0; i < packet->send.packet_length - 1; i++)
              TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 5 + i,
                            packet->send.packet_data[i]);

            /* Get ready for GPS Interrupt */
			intmask = splx(MAX_ILEV);
			pTfpDev->expectedInts = pTfpDev->expectedInts | INT_PACKET;
			splx(intmask);

			TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
			intmask = intmask | INT_PACKET;
			TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug(" SENDGPSPACKET: timeout waiting "
								"for cmd interface\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            /* Interrupt might have already happened, but that's ok */
            retval = ev_receive(INT_PACKET, EV_WAIT | EV_ANY,
                                GPS_TIMEOUT, &event); /* Wait up to 3 sec */
            if (retval)
              packet->receive.packet_length = 0;
            else
              {
                packet->receive.packet_length = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt);
                if (packet->receive.packet_length > 0)
                  packet->receive.packet_id = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt + 1);

                for (i = 0; i < packet->receive.packet_length - 1; i++)
                  {
                    packet->receive.packet_data[i] = TFP_IN_BYTE (pTfpDev->baseAdrsGpsPkt + 2 + i);
                  }
              }
          }

          mu_unlock(pTfpDev->cmdmutex);
          break;

        case BC635_INTHOOK:

/*---------------------------------------------------------------------*/
/* This option will allow the user to have their own ISR executed by   */
/* the tfpDevIsr routine.                                              */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask;

            intmask = splx(MAX_ILEV);
            pTfpDev->inthook = (VOIDFUNCPTR) arg;
            splx(intmask);
          }

          break;

        case BC635_INT_WAITONGPS:

/*---------------------------------------------------------------------*/
/* This option will allow the calling routine to wait on a GPS data    */
/* packet.  This option is for the BC637 only.                         */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask, retval, event;

			pTfpDev->timeout = arg;
            if (pTfpDev->timeout == 0)
				pTfpDev->timeout = EVENT_TIMEOUT;

            intmask = splx(MAX_ILEV);
            pTfpDev->expectedInts = pTfpDev->expectedInts | INT_PACKET;
            splx(intmask);

            TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
            intmask = intmask | INT_PACKET;
            TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            retval = ev_receive(INT_PACKET, EV_WAIT | EV_ANY,
                                pTfpDev->timeout, &event);
            if (retval)
			  {
                bc635PrintDebug(" BC635_INT_WAITONGPS: timeout waiting "
								"for cmd interface\n");
				return(ERROR);
			  }
          }

          break;

        case BC635_INT_WAITONEVENT:

/*---------------------------------------------------------------------*/
/* This option will wait for an event which can be tied to a strobe    */
/* pulse or an external pulse.                                         */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask, retval, event;

			pTfpDev->timeout = arg;
            if (pTfpDev->timeout == 0)
				pTfpDev->timeout = EVENT_TIMEOUT;

            intmask = splx(MAX_ILEV);
            pTfpDev->expectedInts = pTfpDev->expectedInts | BCINT_EVENT;
            splx(intmask);

            TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
            intmask = intmask | BCINT_EVENT;
            TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            retval = ev_receive(BCINT_EVENT, EV_WAIT | EV_ANY,
                                pTfpDev->timeout, &event);
            if (retval)
			  {
				bc635PrintDebug(" BC635_INT_WAITONEVENT: timeout waiting "
								"for cmd interface\n");
				return(ERROR);
			  }
          }

          break;

        case BC635_INT_WAITONSTROBE:

/*---------------------------------------------------------------------*/
/* This option will wait for the preprogrammed Time Coincidence Strobe */
/* Output signal which may be programmed to be activated at a preset   */
/* time.                                                               */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask, retval, event;

			pTfpDev->timeout = arg;
            if (pTfpDev->timeout == 0)
				pTfpDev->timeout = EVENT_TIMEOUT;

            intmask = splx(MAX_ILEV);
            pTfpDev->expectedInts = pTfpDev->expectedInts | INT_STROBE;
            splx(intmask);

            TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
            intmask = intmask | INT_STROBE;
            TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

			/* The Event timeout should be programmable */
            retval = ev_receive(INT_STROBE, EV_WAIT | EV_ANY,
                                pTfpDev->timeout, &event);
            if (retval)
			  {
				bc635PrintDebug(" BC635_INT_WAITONSTROBE: timeout waiting "
								"for cmd interface\n");
				return(ERROR);
			  }
          }

          break;

        case BC635_INT_WAITON1PPS:

/*---------------------------------------------------------------------*/
/* This option will wait for the internal 1 PPS pulse to occur.        */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask, retval, event;

			pTfpDev->timeout = arg;
            if (pTfpDev->timeout == 0)
				pTfpDev->timeout = EVENT_TIMEOUT;

            intmask = splx(MAX_ILEV);
            pTfpDev->expectedInts = pTfpDev->expectedInts | INT_1PPS;
            splx(intmask);

            TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
            intmask = intmask | INT_1PPS;
            TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            retval = ev_receive(INT_1PPS, EV_WAIT | EV_ANY,
                                pTfpDev->timeout, &event);
            if (retval)
			  {
				bc635PrintDebug(" BC635_INT_WAITON1PPS: timeout waiting "
								"for cmd interface\n");
				return(ERROR);
			  }
          }

          break;

        case BC635_INT_WAITONPERIODIC:

/*---------------------------------------------------------------------*/
/* This option will wait for the user programmed Periodic pulse to     */
/* occur.  The period can range from 250 kHz to less than 1 Hz.        */
/*---------------------------------------------------------------------*/
          {
            UINT32  intmask, retval, event;

			pTfpDev->timeout = arg;
            if (pTfpDev->timeout == 0)
				pTfpDev->timeout = EVENT_TIMEOUT;

            intmask = splx(MAX_ILEV);
            pTfpDev->expectedInts = pTfpDev->expectedInts | INT_PERIODIC;
            splx(intmask);

            TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), &intmask);
            intmask = intmask | INT_PERIODIC;
            TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), intmask);

            retval = ev_receive(INT_PERIODIC, EV_WAIT | EV_ANY,
                                pTfpDev->timeout, &event);
            if (retval)
			  {
				bc635PrintDebug(" BC635_INT_WAITONPERIODIC: timeout "
								"waiting for cmd interface\n");
				return(ERROR);
			  }
          }

          break;

        case SELTIMINGMODE:

/*---------------------------------------------------------------------*/
/* This option will select the timing mode of the TFP.  The default    */
/* mode for the BC635 is time code decoding mode and for the BC637 is  */
/* GPS mode.                                                           */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetTmMode (pTfpDev, arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SELTIMEFORMAT:

/*---------------------------------------------------------------------*/
/* This option selects either BCD (decimal) or Binary (Unix) time      */
/* format.  This format affects the TIMEx and EVENTx registers and     */
/* commands.  The default format is Binary (Unix).                     */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetTmFmt (pTfpDev, arg) == ERROR)
            {
              mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
              return (ERROR);
            }

          break;

        case TIMEREQUEST:

/*---------------------------------------------------------------------*/
/* This option latches the current time which can be read later.       */
/*---------------------------------------------------------------------*/
          TFP_OUT_LONG (TFP_REG_TIMEREQ (pTfpDev->baseAdrsRegs), 0);

          break;

        case EVENTREQUEST:

/*---------------------------------------------------------------------*/
/* This option latches the event time which can be read later.  The    */
/* time can also be latched by an external input (if enabled) or the   */
/* programmable periodic interrupt.                                    */
/*---------------------------------------------------------------------*/
          TFP_OUT_LONG (TFP_REG_EVENTREQ (pTfpDev->baseAdrsRegs), 0);
          break;

        case RDTIMETM:

/*---------------------------------------------------------------------*/
/* This option reads the last latched time set by the TIMEREQUEST      */
/* option.  The format depends upon the time format selected.  The     */
/* 'decimal units time' is containded in the TIME0 and TIME1 reg.      */
/* NOTE: The time is not latched by this routine, use RDTIMEREQ to     */
/* latch the current time.                                             */
/*---------------------------------------------------------------------*/
          /* get minor time */
          TFP_IN_LONG (TFP_REG_TIME0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time */
          TFP_IN_LONG (TFP_REG_TIME1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDTIME): maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* stuff in user time struct */
          pTm = (TFP_TM *) arg;

          /* get decimal secs (0-59) */
          pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

          /* get decimal mins (0-59) */
          pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

          /* get decimal hours (0-23) */
          pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

          /* get decimal year-day (0-366) */
          pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

          pTm->tm.tm_year = (TFP_IN_BYTE (pTfpDev->baseAdrsYr) << 8) +
            TFP_IN_BYTE (pTfpDev->baseAdrsYr + 1);

          /* usecs */
          pTm->usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTm->status = TFP_TM0_STATUS (tmMinor);

          break;

        case RDEVENTTM:

/*---------------------------------------------------------------------*/
/* This option reads the last latched time set by the EVENTREQUEST     */
/* option.  The format depends upon the time format selected.  The     */
/* 'decimal units time' is containded in the EVENT0 and EVENT1 reg.    */
/* NOTE: The time is not latched by this routine, use RDEVENTREQ,      */
/* Periodic (heartbeat), or Event Input to latch it.                   */
/*---------------------------------------------------------------------*/
          /* get minor time */
          TFP_IN_LONG (TFP_REG_EVENT0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time */
          TFP_IN_LONG (TFP_REG_EVENT1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDEVENT): maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* stuff in user time struct */
          pTm = (TFP_TM *) arg;

          /* get decimal secs (0-59) */
          pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

          /* get decimal mins (0-59) */
          pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

          /* get decimal hours (0-23) */
          pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

          /* get decimal year-day (0-366) */
          pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

          pTm->tm.tm_year = (TFP_IN_BYTE (pTfpDev->baseAdrsYr) << 8) +
            TFP_IN_BYTE (pTfpDev->baseAdrsYr + 1);

          /* usecs */
          pTm->usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTm->status = TFP_TM0_STATUS (tmMinor);

          break;

          /*
           * get 'unix units' time contained in TIME0/1 major,minor
           * regs
           */
        case RDTIMETMREQ:

/*---------------------------------------------------------------------*/
/* This option will latch and read the current time, thus performing   */
/* both TIMERREQUEST and RDTIMETM functions.  The format depends upon  */
/* the time format selected.  The 'decimal units time' is contained in */
/* the TIME0 AND TIME1 registers.                                      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->timemutex, MU_WAIT, 0);

          /* latch current minor/major time (returned val not used) */
          TFP_IN_LONG (TFP_REG_TIMEREQ (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get minor time */
          TFP_IN_LONG (TFP_REG_TIME0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time */
          TFP_IN_LONG (TFP_REG_TIME1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDTIMEREQ): maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* stuff in user time struct */
          pTm = (TFP_TM *) arg;

          /* get decimal secs (0-59) */
          pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

          /* get decimal mins (0-59) */
          pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

          /* get decimal hours (0-23) */
          pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

          /* get decimal year-day (0-366) */
          pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

          pTm->tm.tm_year = (TFP_IN_BYTE (pTfpDev->baseAdrsYr) << 8) +
            TFP_IN_BYTE (pTfpDev->baseAdrsYr + 1);

          /* usecs */
          pTm->usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTm->status = TFP_TM0_STATUS (tmMinor);

          mu_unlock(pTfpDev->timemutex);

          break;

        case RDEVENTTMREQ:

/*---------------------------------------------------------------------*/
/* This option will latch and read the event time, thus performing 	   */
/* both EVENTREQUEST and RDEVENTTM functions.  The format depends upon */
/* the time format selected.  The 'decimal units time' is contained in */
/* the TIME0 AND TIME1 registers.                                      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->eventmutex, MU_WAIT, 0);

          /* latch current minor/major time (returned val not used) */
          TFP_IN_LONG (TFP_REG_EVENTREQ (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get minor time */
          TFP_IN_LONG (TFP_REG_EVENT0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time */
          TFP_IN_LONG (TFP_REG_EVENT1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDEVENTREQ): maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* stuff in user time struct */
          pTm = (TFP_TM *) arg;

          /* get decimal secs (0-59) */
          pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

          /* get decimal mins (0-59) */
          pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

          /* get decimal hours (0-23) */
          pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

          /* get decimal year-day (0-366) */
          pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

          pTm->tm.tm_year = (TFP_IN_BYTE (pTfpDev->baseAdrsYr) << 8) +
            TFP_IN_BYTE (pTfpDev->baseAdrsYr + 1);

          /* usecs */
          pTm->usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTm->status = TFP_TM0_STATUS (tmMinor);
          mu_unlock(pTfpDev->eventmutex);

          break;

        case RDTIMETV:

/*---------------------------------------------------------------------*/
/* This option reads the current time and sets the Unix time format    */
/* 'tv' values as well.  The 'decimal units time' is contained in the  */
/* TIME0 AND TIME1 registers.                                          */
/* NOTE: The current time is not latched by this routine, use the      */
/* RDTIMEREQ option to do that.                                        */
/*---------------------------------------------------------------------*/
          /* get minor time */
          TFP_IN_LONG (TFP_REG_TIME0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time (unix secs (from 12am 1/1/70) */
          TFP_IN_LONG (TFP_REG_TIME1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDTIMETV):maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* translate to user values */
          pTv = (TFP_TV *) arg;

          /* unix secs (secs from 12am 1/1/70) */
          pTv->tv.tv_sec = tmMajor;

          /* usecs */
          pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTv->status = TFP_TM0_STATUS (tmMinor);

          break;

        case RDEVENTTV:

/*---------------------------------------------------------------------*/
/* This option will read the event time and sets the Unix time format  */
/* 'tv' values as well.  The 'decimal units time' is contained in the  */
/* EVENT0 and EVENT1 registers.                                        */
/* NOTE: The current time is not latched by this routine, use the      */
/* RDEVENTTVREQ option to do that.                                     */
/*---------------------------------------------------------------------*/
          /* get minor time */
          TFP_IN_LONG (TFP_REG_EVENT0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time (unix secs (from 12am 1/1/70) */
          TFP_IN_LONG (TFP_REG_EVENT1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDEVENTTV):maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* translate to user values */
          pTv = (TFP_TV *) arg;

          /* unix secs (secs from 12am 1/1/70) */
          pTv->tv.tv_sec = tmMajor;

          /* usecs */
          pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTv->status = TFP_TM0_STATUS (tmMinor);

          break;

        case RDTIMETVREQ:

/*---------------------------------------------------------------------*/
/* This option will latch and read the current time and sets the Unix  */
/* time format 'tv' values as well.  The 'decimal units time' is con-  */
/* tained in the TIME0 AND TIME1 registers.                            */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->timemutex, MU_WAIT, 0);

          /* latch current minor/major time (returned val not used) */
          TFP_IN_LONG (TFP_REG_TIMEREQ (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get minor time */
          TFP_IN_LONG (TFP_REG_TIME0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time (unix secs (from 12am 1/1/70) */
          TFP_IN_LONG (TFP_REG_TIME1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDTIMETVREQ):maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* translate to user values */
          pTv = (TFP_TV *) arg;

          /* unix secs (secs from 12am 1/1/70) */
          pTv->tv.tv_sec = tmMajor;

          /* usecs */
          pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTv->status = TFP_TM0_STATUS (tmMinor);
          mu_unlock(pTfpDev->timemutex);
          break;

        case RDEVENTTVREQ:

/*---------------------------------------------------------------------*/
/* This option will latch and read the event time and will set the     */
/* Unix time format 'tv' values as well.  The 'decimal units time' is  */
/* contained in the EVENT0 and EVENT1 registers.                       */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->eventmutex, MU_WAIT, 0);

          /* latch current minor/major time (returned val not used) */
          TFP_IN_LONG (TFP_REG_EVENTREQ (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get minor time */
          TFP_IN_LONG (TFP_REG_EVENT0 (pTfpDev->baseAdrsRegs), &tmMinor);

          /* get major time (unix secs (from 12am 1/1/70) */
          TFP_IN_LONG (TFP_REG_EVENT1 (pTfpDev->baseAdrsRegs), &tmMajor);

          /* get minor time (usecs, hundreds nsecs, status) */
          bc635PrintDebug("tfpIoctl(RDEVENTTVREQ):maj,min=x%08X,x%08X\n",
                 tmMajor, tmMinor);

          /* translate to user values */
          pTv = (TFP_TV *) arg;

          /* unix secs (secs from 12am 1/1/70) */
          pTv->tv.tv_sec = tmMajor;

          /* usecs */
          pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

          /* hundreds of nsecs */
          pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

          /* status */
          pTv->status = TFP_TM0_STATUS (tmMinor);
          mu_unlock(pTfpDev->eventmutex);

          break;

        case SETSTROBETM:

/*---------------------------------------------------------------------*/
/* This option programs the Time Coincidence Strobe time using the     */
/* decimal time format.  In this format the Strobe time can be pro-    */
/* grammed from hours through microseconds.  This allow the user to    */
/* program the Stobe to become active up to 48 days after the current  */
/* time.  The Strobe time is also called the Heartbeat time.           */
/* NOTE: The Strobe output must be disabled before programming the     */
/* the Strobe time.  This will prevent spurious Strobe output pulses   */
/* from being generated.                                               */
/*---------------------------------------------------------------------*/
          {
            UINT32  tmp;

            pTm = (TFP_TM *) arg;

            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            tmp = tmp & ~CTRL_STREN;
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
            TFP_OUT_LONG (TFP_REG_MAJSTRB (pTfpDev->baseAdrsRegs),
						  ((pTm->tm.tm_hour << 16) +
						   (pTm->tm.tm_min << 8) +
						   pTm->tm.tm_sec) & 0xffffff);
            TFP_OUT_LONG (TFP_REG_MINSTRB (pTfpDev->baseAdrsRegs),
						  pTm->usec & 0xfffff);
            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            tmp = tmp | CTRL_STREN;
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
          }

          break;

        case SETSTROBETV:

/*---------------------------------------------------------------------*/
/* This option programs the Time Coincidence Strobe time using the     */
/* Unix time 'tv' or binary format.  In this format only the least     */
/* significant 22 bits of the major time are used in addition to the   */
/* microseconds value of the second register.  The Strobe time is also */
/* called the Heartbeat time.                                          */
/* NOTE: The Strobe output must be disabled before programming the     */
/* the Strobe time.  This will prevent spurious Strobe output pulses   */
/* from being generated.                                               */
/*---------------------------------------------------------------------*/
          {
            UINT32  tmp;

            pTv = (TFP_TV *) arg;

            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            tmp = tmp & ~CTRL_STREN;
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
            TFP_OUT_LONG (TFP_REG_MAJSTRB (pTfpDev->baseAdrsRegs),
						  pTv->tv.tv_sec);
            TFP_OUT_LONG (TFP_REG_MINSTRB (pTfpDev->baseAdrsRegs),
						  pTv->tv.tv_usec & 0xfffff);
            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            tmp = tmp | CTRL_STREN;
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
          }

          break;

        case SELTCFORMAT:
/*---------------------------------------------------------------------*/
/* This option selects the time code format for the TFP Timing Mode 0  */
/* input.  Options are: 'A'=IRIG-A, 'B'=IRIG-B, 'I'=IEEE 1344 and      */
/* 'N'=NASA36 format.  The default is IRIG-B.                          */
/* NOTE: This option is active only the Timing Mode is set to Time     */
/* Code receiving mode (0x00).                                         */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetTcFmt (pTfpDev, arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }

          mu_unlock(pTfpDev->cmdmutex);

          break;


        case SELTCMOD:

/*---------------------------------------------------------------------*/
/* This option selects the time code modulation format for the TFP     */
/* Timing Mode 0 input.  Options are: 'M' = Amplitude modulated sine   */
/* wave and 'D' = Pulse code modulation (DC shift level).  The default */
/* is 'M'.                                                             */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetTcm (pTfpDev, arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETTIMETM:

/*---------------------------------------------------------------------*/
/* This option 'seeds' the BC635/7 with an initial time in BCD (deci-  */
/* mal) format.  This option is normally used when the timing mode is  */
/* set to Free Running Mode (1) or 1 PPS mode (2).  The TFP derives    */
/* its major time externally from timing reference signals in modes 0  */
/* and 6 and internally from the realtime clock (RTC) in mode 3.  The  */
/* time written by this command, if used in modes 0, 3, or 6 will be   */
/* overwritten when the selected source is providing a valid time to   */
/* to the TFP.                                                         */
/* NOTE: The time loaded by this command will not be readable until    */
/* the one second epoch following the load.  There is a possibility    */
/* that the TFP will have incremented the time during the load.  To    */
/* prevent ambiguities in the time, the user must issue this command   */
/* in advance of the 800 millisecond point within the one second epoch */
/* with reference to the current epoch.                                */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetDecTime (pTfpDev, (struct tm *) arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETTIMETV:

/*---------------------------------------------------------------------*/
/* This option used the Dual-ported RAM to set the user time.  A Hard- */
/* ware handshake is used to communication to perform synchronization. */
/*---------------------------------------------------------------------*/
		  pTv = (TFP_TV *) arg;
		  mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

		  /* stuff cmd arg bytes in tfp input area */
		  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETTIME);

		  /* set time */
		  TFP_OUT_LONG (pTfpDev->baseAdrsInput + 1, (UINT32) pTv->tv.tv_sec);

		  /* tell tfp to execute and complete command */
		  if (tfpCmdSend (pTfpDev) == ERROR)
			{
			  bc635PrintDebug("tfpIoctl(SETTIMETV): Timeout waiting "
							  "for tfpCmdSend interface\n");
			  mu_unlock(pTfpDev->cmdmutex);
			  return (ERROR);
			}
		  mu_unlock(pTfpDev->cmdmutex);

		  break;

        case SETYEAR:

/*---------------------------------------------------------------------*/
/* This option used the Dual-ported RAM to set the year.  A Hardware   */
/* handshake is used to communication to perform synchronization.      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          if (tfpCmdSetYr (pTfpDev, arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETPROPDELAY:

/*---------------------------------------------------------------------*/
/* This option used the Dual-ported RAM to set the Propagation Delay   */
/* offset.  A hardware handshake is used to communication to perform   */
/* synchronization.                                                    */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          if (tfpCmdSetDelay (pTfpDev, arg) == ERROR)
            {
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);
          break;

        case SETPERIODIC:
          {
            struct periodic *Per = (struct periodic *) arg;

			Per->n1 = MAX(2, Per->n1);		/* Can't be 0 or 1 */
			Per->n2 = MAX(2, Per->n2);
			
            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETPERIODIC);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, Per->sync1pps);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, Per->n1 >> 8);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, Per->n1);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 4, Per->n2 >> 8);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 5, Per->n2);

            bc635PrintDebug("SETPERIODIC: pIn = 0x%x, 0x%x,0x%x,0x%x,0x%x,0x%x\n",
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput),
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput + 1),
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput + 2),
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput + 3),
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput + 4),
                   TFP_IN_BYTE (pTfpDev->baseAdrsInput + 5));

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP SETPERIODIC Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }
            mu_unlock(pTfpDev->cmdmutex);
          }
          break;

        case SELFREQUENCYOUT:

/*---------------------------------------------------------------------*/
/* This option used to set the output frequency to 1, 5, or 10 MHz.    */
/*---------------------------------------------------------------------*/
          {
            UINT32  tmp;

            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            switch (arg)
              {
              case FREQ_10MHZ:
                tmp = (tmp & ~CTRL_FMASK) | CTRL_F10;
                break;
              case FREQ_5MHZ:
                tmp = (tmp & ~CTRL_FMASK) | CTRL_F5;
                break;
              case FREQ_1MHZ:
                tmp = (tmp & ~CTRL_FMASK) | CTRL_F1;
                break;
              default:
                bc635PrintDebug("TFP SELFREQUENCYOUT Failed - bad arg\n");
                return (ERROR);
              }
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
          }

          break;

        case CONTROLEVENT:

/*---------------------------------------------------------------------*/
/* This option used to select various Event input options.  The Event  */
/* Sense (EVSENSE) option allows the selection of triggering on a      */
/* rising or falling edge of a pulse.  The Event Enable (EVENTEN)      */
/* enables or disables the external event capture.  The Event Lockout  */
/* option (LOCKEN) disables all event sources.  The Event Source opt-  */
/* ion is used to switch between external input or the programmable    */
/* periodic input.                                                     */
/*---------------------------------------------------------------------*/
          {
            UINT32  tmp;

            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
			if (arg & 0x10000000)
			  {
				tmp = (tmp & 0xFFFFFF00) | (arg & 0xFF);
			  }
			else switch (arg)
              {
              case EVENT_RISING:
                tmp = tmp | CTRL_EVSENSE;	/* Opposite of 12083 PCI board */
                break;
              case EVENT_FALLING:
                tmp = tmp & ~CTRL_EVSENSE;	/* Opposite of 12083 PCI board */
                break;
              case CAPLOCK_ENABLE:
                tmp = tmp | CTRL_LOCKEN;
                break;
              case CAPLOCK_DISABLE:
                tmp = tmp & ~CTRL_LOCKEN;
                break;
              case EVCAP_ENABLE:
                tmp = tmp | CTRL_EVENTEN;
                break;
              case EVCAP_DISABLE:
                tmp = tmp & ~CTRL_EVENTEN;
                break;
              case EVCAP_PERIODIC:
                tmp = tmp | CTRL_EVSOURCE;
                break;
              case EVCAP_EVENT:
                tmp = tmp & ~CTRL_EVSOURCE;
                break;
              default:
                bc635PrintDebug("TFP CONTROLEVENT Failed - bad arg\n");
                return (ERROR);
              }

            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
			bc635PrintDebug("TFP CONTROLEVENT - Control Register: 0x%X\n",
							tmp);
          }
          break;

        case CONTROLSTROBE:

/*---------------------------------------------------------------------*/
/* This option used to select various Strobe (Heartbeat) options.  The */
/* Strobe Enable (STREN) option enables or disables strobe pulse out-  */
/* put.  The Strobe Mode (STRMODE) option allow use of the Major and   */
/* Minor time values or only the minor time.  When the minor time      */
/* alone is used a strobe pulse is produced once a second.             */
/*---------------------------------------------------------------------*/
          {
            UINT32  tmp;

            TFP_IN_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), &tmp);
            switch (arg)
              {
              case STROBE_SECUS:
                tmp = tmp & ~CTRL_STRMODE;
                break;
              case STROBE_USONLY:
                tmp = tmp | CTRL_STRMODE;
                break;
              case STROBE_ENABLE:
                tmp = tmp | CTRL_STREN;
                break;
              case STROBE_DISABLE:
                tmp = tmp & ~CTRL_STREN;
                break;
              default:
                bc635PrintDebug("TFP CONTROLSTROBE Failed - bad arg\n");
                return (ERROR);
              }
            TFP_OUT_LONG (TFP_REG_CONTROL (pTfpDev->baseAdrsRegs), tmp);
          }
          break;

        case CAPUNLOCK:
          TFP_OUT_LONG (TFP_REG_UNLOCK (pTfpDev->baseAdrsRegs), 0);
          break;

        case RDINTSTAT:
          TFP_IN_LONG (TFP_REG_INTSTAT (pTfpDev->baseAdrsRegs), (UINT32 *) arg);
          break;

        case SETINTSTAT:
          TFP_OUT_LONG (TFP_REG_INTSTAT (pTfpDev->baseAdrsRegs), arg);
          break;

        case RDINTMASK:
          TFP_IN_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), (UINT32 *) arg);
          break;

        case SETINTMASK:
          TFP_OUT_LONG (TFP_REG_INTMASK (pTfpDev->baseAdrsRegs), arg);
          break;

#ifdef not_def
        case RDFIRMVERSION:
          {
            struct gd_fwversion *Ver = (struct gd_fwversion *) arg;

            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_GETDATA);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, GETDATA_VERSION);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP RDFIRMVERSION Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            Ver->vmajor = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 1);
            Ver->vminor = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 2);
            Ver->month = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 3);
            Ver->day = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 4);
            Ver->year = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 5);
            mu_unlock(pTfpDev->cmdmutex);
          }

          break;
#endif

        case CONTROLTIMEBASE:

/*---------------------------------------------------------------------*/
/* This option used the Dual-ported RAM to set various options.  The   */
/* clock option 'I' allows the internal 10 MHz Oscillator drive the    */
/* TFP frequency.  The clock option 'E' allows allow an external input */
/* to drive the TFP frequency.  The JAMSYNC option allow the TFP to    */
/* perform automatic jam-syncs.  This allow continous synchronization  */
/* between the IRIG-B generator and the time code receiver.  The dis-  */
/* cipline option allows turns on or off the set discipline gain for   */
/* an external oscillator.  The FORCE_JAMSYNC option will force a jam  */
/* sync regardless of whether they are being done automatically.       */
/*---------------------------------------------------------------------*/
		  mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
		  switch (arg)
			{
			case CLOCK_INTERNAL:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SELCLOCK);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, SELCLOCK_INT);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case CLOCK_EXTERNAL:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SELCLOCK);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, SELCLOCK_EXT);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case JAMSYNC_ENABLE:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_JAMSYNCENABLE);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, JAMSYNC_ENABLE);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case JAMSYNC_DISABLE:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_JAMSYNCENABLE);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, JAMSYNC_DISABLE);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case DISCIPLINE_ENABLE:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_DISCENABLE);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, DISC_EN);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}

			  break;

			case DISCIPLINE_DISABLE:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_DISCENABLE);
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, DISC_DIS);

			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case FORCE_JAMSYNC:
			  TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_FORCEJAMSYNC);
			  if (tfpCmdSend (pTfpDev) == ERROR)
				{
				  bc635PrintDebug("TFP CONTROLTIMEBASE Failed\n");
				  mu_unlock(pTfpDev->cmdmutex);
				  return (ERROR);
				}
			  break;

			case CLEAR_USEC:
			  break;

			default:
			  return (ERROR);
			}

		  mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETDAC:

/*---------------------------------------------------------------------*/
/* This option controls the crystal oscillator frequency by using a    */
/* 16-bit DAC to generate the controlling voltage.  The user can load  */
/* a value into the D/A and fine tune the TFP frequency base in the    */
/* free running mode.  The D/A output voltages range from 0.5V (0x0)   */
/* to 4.5V (0xFFFF).  The default value is oscillator dependent and is */
/* stored in the EEPROM.  The REQUEST TFP DATA (0x19) command can be   */
/* used to query the factory set value.                                */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETDAC);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg >> 8);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETDAC Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case GETDATA:

/*---------------------------------------------------------------------*/
/* This option used to query the DPRAM for request value(s).           */
/*---------------------------------------------------------------------*/
          {
            struct getdata_t *getdata = (struct getdata_t *) arg;
            int     get_data_len = 0;
            unsigned char *getdataptr = (unsigned char *) & getdata->data.tmode;
            int     i;

            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_GETDATA);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, getdata->arg);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP GETDATA Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            bc635PrintDebug("TFP GETDATA arg = %x\n", getdata->arg);
            switch (getdata->arg)
              {
              case GETDATA_MODE:
                get_data_len = sizeof (getdata->data.tmode);
                break;

              case GETDATA_TFORMAT:
                get_data_len = sizeof (getdata->data.tformat);
                break;

              case GETDATA_YEAR:
                get_data_len = sizeof (getdata->data.year);
                break;

              case GETDATA_PER:
                get_data_len = sizeof (getdata->data.period);
                break;

              case GETDATA_TCFMT:
                get_data_len = sizeof (getdata->data.tcformat);
                break;

              case GETDATA_TCMOD:
                get_data_len = sizeof (getdata->data.tcmod);
                break;

              case GETDATA_TOFF:
                get_data_len = sizeof (getdata->data.offset);
                break;

              case GETDATA_UTCINFO:
                get_data_len = sizeof (getdata->data.utc);
                break;

              case GETDATA_TCOUTFMT:
                get_data_len = sizeof (getdata->data.tcoutfmt);
                break;

              case GETDATA_TCGENOFF:
                get_data_len = sizeof (getdata->data.genoff);
                break;

              case GETDATA_LOCTMOFF:
                get_data_len = sizeof (getdata->data.locoff);
                break;

              case GETDATA_LEAPSEC:
                get_data_len = sizeof (getdata->data.leapsec);
                break;

              case GETDATA_FWVER:
                get_data_len = sizeof (getdata->data.version);
                break;

              case GETDATA_CLKSRC:
                get_data_len = sizeof (getdata->data.clksrc);
                break;

              case GETDATA_JAMSC:
                get_data_len = sizeof (getdata->data.jsctl);
                break;

              case GETDATA_OSCCTL:
                get_data_len = sizeof (getdata->data.ocsctl);
                break;

              case GETDATA_DAVAL:
                get_data_len = sizeof (getdata->data.davalue);
                break;

              case GETDATA_BATTSTAT:
                get_data_len = sizeof (getdata->data.battstat);
                break;

              case GETDATA_CLKVAL:
                get_data_len = sizeof (getdata->data.clkval);
                break;

              case GETDATA_DTFW:
                get_data_len = sizeof (getdata->data.dtfw);
                break;

              case GETDATA_ASSEMB:
                get_data_len = sizeof (getdata->data.assemb);
                break;

              case GETDATA_HWFAB:
                get_data_len = sizeof (getdata->data.hwfab);
                break;

              case GETDATA_TFPMODEL:
                get_data_len = sizeof (getdata->data.tfpm);
                break;

              case GETDATA_SERIAL:
                get_data_len = sizeof (getdata->data.serialnum);
                break;

              default:
                get_data_len = 0;
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            for (i = 0; i < get_data_len; i++)
              {
                getdataptr[i] = TFP_IN_BYTE (pTfpDev->baseAdrsOutput + 1 + i);
                bc635PrintDebug("TFP GETDATA [%d] = %x\n", i, getdataptr[i]);
              }
            bc635PrintDebug("TFP GETDATA Complete\n");
          }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SOFTWARERESET:

/*---------------------------------------------------------------------*/
/* This option uses the Dual-ported RAM to reset the TFP to a power-on */
/* state.  NOTE: The device registers are unaffected by this.          */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SOFTRESET);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, SW_RESET);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SOFTWARERESET Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          tm_wkafter(1);
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETTCOUTFMT:

/*---------------------------------------------------------------------*/
/* This option sets the time code output format on the J1 connector.   */
/* The options are: 'B'=IRIG-B and 'I'=IEEE-1344.                      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETTCOUTFMT);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETTCOUTFMT Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETGENTMOFFSET:

/*---------------------------------------------------------------------*/
/* This option sets the time code generator output time offset.  This  */
/* adds or subtracts an offset to time code generator output.          */
/*---------------------------------------------------------------------*/
          {
            struct tcgenoffset *tcgoff = (struct tcgenoffset *) arg;

            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETGENTMOFF);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, tcgoff->offset >> 8);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, tcgoff->offset);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, tcgoff->half_hour);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP SETGENTMOFFSET Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }
          }

          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETLOCTMOFFSET:

/*---------------------------------------------------------------------*/
/* This option sets the local time offset by adding or subtracting an  */
/* offset to the TFP time.  This command affects the TFP time only.    */
/*---------------------------------------------------------------------*/
          {
            struct loctmoffset *locoff = (struct loctmoffset *) arg;

            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);

            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETLOCTMOFF);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, locoff->offset >> 8);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, locoff->offset);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, locoff->half_hour);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP SETLOCTMOFFSET Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }
            mu_unlock(pTfpDev->cmdmutex);
          }

          break;

        case SETLEAPSECEVENT:

/*---------------------------------------------------------------------*/
/* This option inserts or deletes a leap second into the TFP local     */
/* time and output.  The time is represented as Unix time.             */
/*---------------------------------------------------------------------*/
          {
            struct leapseconds *lpsec = (struct leapseconds *) arg;

            mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETLEAPSEC);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, lpsec->ls_flag);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, lpsec->leap_tm >> 24);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, lpsec->leap_tm >> 16);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 4, lpsec->leap_tm >> 8);
            TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 5, lpsec->leap_tm);

            if (tfpCmdSend (pTfpDev) == ERROR)
              {
                bc635PrintDebug("TFP SETLEAPSECEVENT Failed\n");
                mu_unlock(pTfpDev->cmdmutex);
                return (ERROR);
              }

            mu_unlock(pTfpDev->cmdmutex);
          }

          break;

        case SETDISCGAIN:

/*---------------------------------------------------------------------*/
/* This option sets the gain and sense of the disciplining process to  */
/* be set by the user.  Increasing the gain increases the VCO clock    */
/* source freqency.  Decreasing the gain with negative values de-      */
/* creses the frequency.  This feature is valuable to the user who is  */
/* attempting to discipline an external oscillator using the TFP.      */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETDISCGAIN);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg >> 8);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETDISGAIN Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETCLKVAL:

/*---------------------------------------------------------------------*/
/* This option advances or retards the TFP clock that is used to slew  */
/* the timing of the 1 PPS pulse.  This command will allow the user to */
/* speed up or slow down the on-board oscillator while the TFP is in   */
/* flywheel or Freewheel mode.  The TFP adjusts up to 100 msec per     */
/* second.  Each count is 10 microseconds.                             */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETCLKVAL);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg >> 24);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, arg >> 16);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, arg >> 8);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 4, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETCLKVAL Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETGPSTMFMT:

/*---------------------------------------------------------------------*/
/* This option allows the user to select between GPS time and UTC when */
/* using Timing Mode 6.  The options are 0=UTC and 1=GPS.  The default */
/* is UTC.                                                             */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_GPSTIMEFORMAT);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg >> 24);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 2, arg >> 16);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 3, arg >> 8);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 4, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETGPSTMFMT Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }

          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETGPSMDFLG:

/*---------------------------------------------------------------------*/
/* This option directs the GPS receiver to static operational or sta-  */
/* tion mode after the tracing bit resets to zero.  This command al-   */
/* lows the user to disable this feature.                              */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETGPSMDFLG);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETGPSMDFLG Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETLOCTMFLG:

/*---------------------------------------------------------------------*/
/* This option set the Local time (not ZULU) mode.  If the flag is     */
/* enabled, the TFP adjusts its time to reflect the local time.        */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETLOCTMFLG);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETLOCTMFLG Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }

          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SETYRINCFLG:

/*---------------------------------------------------------------------*/
/* This option turns on the year auto-incrementing flag.  The year     */
/* value is stored in EEPROM at the beginning of each year.  Default   */
/* is enabled.                                                         */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SETYRINCFLG);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput + 1, arg);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SETYRINCFLG Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }

          mu_unlock(pTfpDev->cmdmutex);

          break;

        case SYNCRTC:

/*---------------------------------------------------------------------*/
/* This option forces a one time synchronization of the TFP real-time  */
/* clock (RTC) to the current TFP time.  This command is only valid    */
/* for modes 1, 2, or 6.  The TFP cannot be in mode 3.                 */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_SYNCRTC);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP SYNCRTC Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        case DISCBATT:

/*---------------------------------------------------------------------*/
/* This option disconnects the battery power to the RTC during the     */
/* time that the power is removed.  This keeps the battery from dis-   */
/* charging while in storage.  The next time the board is powered up,  */
/* the battery manager will automatically resume constant battery      */
/* power to the RTC.                                                   */
/*---------------------------------------------------------------------*/
          mu_lock(pTfpDev->cmdmutex, MU_WAIT, 0);
          TFP_OUT_BYTE (pTfpDev->baseAdrsInput, COM_DISCBATTRTC);

          if (tfpCmdSend (pTfpDev) == ERROR)
            {
              bc635PrintDebug("TFP DISCBATT Failed\n");
              mu_unlock(pTfpDev->cmdmutex);
              return (ERROR);
            }
          mu_unlock(pTfpDev->cmdmutex);

          break;

        default:
          return (ERROR);
        }                       /* switch */

      return (OK);
  }

/******************************************************************************
*
* tfpCmdSetTmMode - command tfp to set 'timing mode'.
*
* The 'mode' argument must be set to the timing mode desired
* and must be one of the following:
*       define MODE_TIMECODE            0
*       define MODE_FREERUN             1
*       define MODE_EXT1PPS             2
*       define MODE_RTC                 3
*       define MODE_GPS                 6
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdSetTmMode (TFP_DEV * pDev, int mode)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMINGMODE);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) mode);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetTmMode: timed out waiting for "
							"cmd interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetTmMode: cmd complete, mode=%d\n",
                                       mode);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetTmFmt - command tfp to set 'time register format'
*
* The 'fmt' argument must be set to one of the following:
*       define TIME_DECIMAL     0
*       define TIME_BINARY      1
*
* Upon return, the user must set/read 'major time' according to the format set.
*
* NOTE: In the Programmer's Guide, the term 'binary time' is used to refer
*       to the time registers when they are formatted with unix time.
*       By unix time, they mean 'seconds since 1970'.
*
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdSetTmFmt(TFP_DEV * pDev, int fmt)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMEFORMAT);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) fmt);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetTmFmt: timeout waiting for cmd "
							"interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetTmFmt: cmd complete, "
								"fmt=%02X\n", fmt);

        return (OK);

  }


/******************************************************************************
*
* tfpCmdSetTcFmt - command tfp to set 'timecode format'
*
* The 'fmt' argument must be set to one of the following:
#       define TC_IRIGA         'A'
#       define TC_IRIGB         'B'
#       define TC_IEEE          'I'

* RETURNS: OK always.
*/

  STATUS  tfpCmdSetTcFmt(TFP_DEV * pDev, int fmt)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTCFORMAT);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) fmt);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetTcFmt: timeout waiting for cmd "
							"interface\n");
            return (ERROR);
        }

        bc635PrintDebug("tfpCmdSetTcFmt: cmd complete, fmt=%02X\n", fmt);

        return (OK);
  }


/******************************************************************************
*
* tfpCmdSetDecTime - set major time using 'decimal' format
*
* The following members of the 'tm' user struct are used:
*    tm_year: 1970 - 2050
*    tm_yday: 0 - 366
*    tm_hour: 0 - 23
*    tm_min: 0 - 59
*    tm_sec: 0 - 59
* Other members are not used.
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdSetDecTime (TFP_DEV * pDev, struct tm * pTm)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIME);

        /* msb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, MSB ((UINT16) pTm->tm_year));
        /* lsb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, LSB ((UINT16) pTm->tm_year));

        /* msb - yday */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 3, MSB ((UINT16) pTm->tm_yday));
        /* lsb - yday */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 4, LSB ((UINT16) pTm->tm_yday));

        /* lsb - hour (0-23) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 5, LSB ((UINT16) pTm->tm_hour));

        /* lsb - min (0-59) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 6, LSB ((UINT16) pTm->tm_min));

        /* lsb - sec (0-59) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 7, LSB ((UINT16) pTm->tm_sec));

        bc635PrintDebug("pTm->tm_min=%d\n", LSB ((UINT16) pTm->tm_min));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetDecTime: timeout waiting for cmd "
							"interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetDecTime: cmd complete\n");

        return (OK);
  }


/******************************************************************************
*
* tfpCmdSetYr - set year
*
* Valid range for 'year' is 1970 - 2050
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdSetYr( TFP_DEV * pDev, int year)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETYEAR);

        /* msb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, MSB ((UINT16) year));
        /* lsb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, LSB ((UINT16) year));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetYr: timeout waiting for cmd interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetYr: cmd complete, year=%d\n", year);

        return (OK);
  }


/******************************************************************************
*
* tfpCmdSetTcm - command tfp to set 'timecode modulation format'
*
* The 'type' argument must be set to one of the following modulation types:
*    TFP_TCM_AM      0x4D    * expect 'amplitude modulated' format
*    TFP_TCM_PCM     0x44    * expect 'pulse code modulated' format

* RETURNS: OK always.
*/

  STATUS  tfpCmdSetTcm(TFP_DEV * pDev, int type)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTCMOD);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) type);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetTcm: timeout waiting for cmd interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetTcm: cmd complete, type=%02X\n",
                                       type);

        return (OK);

  }


/******************************************************************************
*
* tfpCmdSetDelay - command tfp to set 'propagation delay'
*
* The 'offset' argument is in units of 100ns and may be positive
* or negative.

* RETURNS: OK always.
*/

  STATUS  tfpCmdSetDelay(TFP_DEV * pDev, int offset)
  {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMINGOFFSET);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) LLSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, (UINT8) LNLSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 3, (UINT8) LNMSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 4, (UINT8) LMSB (offset));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSetDelay: timeout waiting for cmd interface\n");
            return (ERROR);
        }

                bc635PrintDebug("tfpCmdSetDelay: cmd complete, offset=%02X\n",
                                       offset);

        return (OK);
  }



/******************************************************************************
*
* tfpCmdGetTfpModel - command tfp to return 'tfp model' characters
*
* The model chars are TFP_MODEL_LEN(9) long and can be treated
* as an unterminated string. The 'model' chars are placed in
* caller's 'buf' arg as a terminated string.
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdGetTfpModel(TFP_DEV * pDev, char *buf)
  {

        if (tfpCmdReqDataSend (pDev, COM_REQTFPMODEL) == ERROR)
        {
            bc635PrintDebug("tfpCmdTfpModel: tfpCmdReqDataSend failed\n");
            return (ERROR);
        }

        /* get string (skip over request code in first location) */
        tfpGetOutBytes (pDev->baseAdrsOutput + 1, buf, TFP_MODEL_LEN);

        /* string is unterminated, so terminate it */
        *(buf + TFP_MODEL_LEN) = EOS;

        bc635PrintDebug("tfpCmdTfpModel: str=%s\n", (int) buf);

        return (OK);

  }


/******************************************************************************
*
* tfpCmdReqDataSend - command tfp to exexute a 'data request' type of command
*
* This function is used to send a 'tfp request data (cmd id=0x19)' command.
* The 'reqDataCode' arg identifies the specific data that the tfp is to return.
*
* RETURNS: OK always.
*/

  STATUS  tfpCmdReqDataSend(TFP_DEV * pDev, UINT8 reqDataCode)
  {
        UINT8   val;


        /* issue 'req data' cmd */

                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_GETDATA);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, reqDataCode);

        if      (tfpCmdSend (pDev) == ERROR)
        {
            bc635PrintDebug("tfpCmdSendReqData: timeout waiting for cmd interface\n");
            return (ERROR);
        }

        /* verify request code located in first location */
                val = TFP_IN_BYTE (pDev->baseAdrsOutput);

        if (val != reqDataCode)
        {
            bc635PrintDebug("tfpCmdSendReqData: bad req code read\n");
            return (ERROR);
        }

        return (OK);
  }



/******************************************************************************
*
* tfpCmdSend - send command to TFP for execution
*
* The caller is to fill the 'input area' with a set of command
* byte values for some tfp command. This function is then called
* to perform the command execution protocol. Upon return, the
* command is complete.
*
* RETURNS: OK always.
*/

  STATUS tfpCmdSend (TFP_DEV * pDev)
  {
        ULONG   val;
        FAST int i = 0;
        const int secsTimeOut = sysClkRateGet () * 5;


        /* clear bit 0 of 'ack reg by writing '1' to reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CLR);

        /* inform tfp of command is waiting by writing 0x80 to 'ack' reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CMDRDY);

        /* wait for tfp to set bit 0 of 'ack' reg */
        while (i++ < secsTimeOut)
        {
            TFP_IN_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), &val);

            if ((val & TFP_ACK_COMPLETE) > 0)
                break;

            tm_wkafter(1);
        }

        /* clear bit 0 of 'ack reg by writing '1' to reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CLR);

        if (i == secsTimeOut)
        {
            bc635PrintDebug("tfpCmdSend: timeout\n");
            return (ERROR);
        }

        return (OK);

  }



/******************************************************************************
*
* tfpGetOutBytes - copy tfp 'output area' bytes to local address
*
* This routine copies bytes the tfp has returned to the 'ouput area'
* as the result of some 'data request' command to a local address.
*
* outputAdrs = the address as seen from the cpu of the tfp 'output area'
*              to start copying from.
*
* pDest = the address to copy 'output area' bytes to.
*
* len = the number of bytes to copy.
*
* RETURNS: number of bytes copied (should always be 'len')
*/

  int tfpGetOutBytes(UINT32 outputAdrs, unsigned char * pDest, int len)
  {
        unsigned char *p = pDest;
        UINT32  adrs = outputAdrs;
        int     i;

        for     (i = 0; i < len; i++)
                   *p++ = TFP_IN_BYTE (adrs++);

        return (len);
  }

#ifdef __cplusplus
}

#endif

#endif                          /* __INCbc635c */

