/* @(#) pSOSystem PowerPC/V2.5.0: apps/psosdemo/demo.c (&U&) 4.1 98/10/15 15:16:23 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  apps/demo25/demo.c                                       */
/*   DATE:    98/10/15                                                 */
/*   PURPOSE: A sample application that demonstrates the new features  */
/*            of pSOS+ V2.5.0 kernel.                                  */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1997, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/***********************************************************************/

/*
 sample.c  - This is an interactive test program demonstrating the
             functionality of the bc635/637 VxWorks Device Driver.

             It is intended as example code, to assist a programmer in
             developing their own code using the driver.

             This test program lacks proper error checking, and also
             does not prevent erroneous actions.  (An example of an
             erroneous action would be to read a decimal time format when
             the board is in binary time mode).
*/
/*
***********************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
***********************************************************************
*/

#include "sys_conf.h"
#include "bsp.h"
#include <bspfuncs.h>
#include "powerpc/isrppc.h"
#include "../devices/pci/pcihdr.h"
#include <stdio.h>
#include <time.h>
#include "bc635_mnr.h"
#include "bc635P_mnr.h"
#include "generalcore.h"
#include "integer_type.h"

#define NOERR               0

#if (BSP_CPUFAMILY == FAMILY_960)
#define SSTK_SIZE    6144
#define USTK_SIZE    0
#else
#define SSTK_SIZE    4096
#define USTK_SIZE    256
#endif

/*---------------------------------------------------------------------*/
/* Parameters                                                          */
/*---------------------------------------------------------------------*/
#define    MAX_HANDLERS     0xFF

#define MODE_RTC        3
#define RC_OK           1
#define RC_ERROR        (-1)

/*---------------------------------------------------------------------*/
/* Function Prototypes                                                 */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/* Global variables                                                    */
/*---------------------------------------------------------------------*/
/* TFP_DEV             *pTfpDev; /* /* moved this pointer dec into bc635.c */
struct getdata_t    get;

/*---------------------------------------------------------------------*/
/* External variables                                                  */
/*---------------------------------------------------------------------*/
extern UINT32       bc635Debug;


/**********************************************************************/
/* Description: Set Oper Mode */
/**********************************************************************/
void
pci_mode ()
{
    int     oper_mode = 0;

    printf ("\n\n\n\t    Select Operational Mode:\n\n");
    printf ("\t0. Time Code Mode\n");
    printf ("\t1. Free Running Mode\n");
    printf ("\t2. External 1PPS\n");
    printf ("\t3. Real Time Clock Mode\n");
    printf ("\t4. GPS Mode\n\n");
    printf ("\t   Select: ");
    scanf ("%i", &oper_mode);

    switch (oper_mode)
      {
      case 0:
	/*	tfpIoctl (pTfpDev, SELTIMINGMODE, (int) MODE_TIMECODE); */
	tfpIoctl (SELTIMINGMODE, (int) MODE_TIMECODE);
        break;                  /* Timing Mode: time code */

      case 1:
	/*        tfpIoctl (pTfpDev, SELTIMINGMODE, (int) MODE_FREERUN); */
        tfpIoctl (SELTIMINGMODE, (int) MODE_FREERUN);
        break;                  /* Timing Mode: Free Running */

      case 2:
	/*        tfpIoctl (pTfpDev, SELTIMINGMODE, (int) MODE_EXT1PPS); */
        tfpIoctl (SELTIMINGMODE, (int) MODE_EXT1PPS);
        break;                  /* Timing Mode: 1PPS */

      case 3:
	/*        tfpIoctl (pTfpDev, SELTIMINGMODE, (int) MODE_RTC); */
        tfpIoctl (SELTIMINGMODE, (int) MODE_RTC);
        break;                  /* Timing Mode: RTC */

      case 4:
	/*        tfpIoctl (pTfpDev, SELTIMINGMODE, (int) MODE_GPS); */
        tfpIoctl (SELTIMINGMODE, (int) MODE_GPS);
        break;                  /* Timing Mode: GPS */

      default:
        printf ("Error setting operational mode!!!!");
        break;
      }
}

/**********************************************************************/
/* Description : Set Time Code Format */
/**********************************************************************/
void
pci_time_code ()
{
    int     tc_fmt;
    int     tc_mod;

    printf ("\n\n\n\t    Select Time Code Format:\n\n");
    printf ("\t1. IRIG A\n");
    printf ("\t2. IRIG B\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &tc_fmt);
    switch (tc_fmt)
    {
        case 1:
	  /*            tfpIoctl (pTfpDev, SELTCFORMAT, TC_IRIGA);  /* IRIG A time code */
            tfpIoctl (SELTCFORMAT, TC_IRIGA);  /* IRIG A time code */
            break;
        case 2:
	  /*            tfpIoctl (pTfpDev, SELTCFORMAT, TC_IRIGB);  /* IRIG B time code */
            tfpIoctl (SELTCFORMAT, TC_IRIGB);  /* IRIG B time code */
            break;
        default:
            printf ("Error Setting Time Code Format!");
            break;
    }

    printf ("\n\n\n\t    Select Time Code Modulation:\n\n");
    printf ("\t1. Modulated\n");
    printf ("\t2. DC Level Shift\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &tc_mod);
    switch (tc_mod)
    {
        case 1:
	  /*            tfpIoctl (pTfpDev, SELTCMOD, MOD_AM);       /* Modulated */
            tfpIoctl (SELTCMOD, MOD_AM);       /* Modulated */
            break;
        case 2:
	  /*            tfpIoctl (pTfpDev, SELTCMOD, MOD_DCLS);     /* DCLS */
            tfpIoctl (SELTCMOD, MOD_DCLS);     /* DCLS */
            break;
        default:
            printf ("Error Setting Time Code Modulation!");
            break;
    }
}

/**********************************************************************/
/* Description : Set Output Frequency */
/**********************************************************************/
void
pci_out_freq ()
{
    int     out_freq = 3;

    printf ("\n\n\n\t    Select Output Frequency:\n\n");
    printf ("\t1. 1  MHz\n");
    printf ("\t2. 5  MHz\n");
    printf ("\t3. 10 MHz\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &out_freq);
    switch (out_freq)
    {
        case 1:
	  /*            tfpIoctl (pTfpDev, SELFREQUENCYOUT, FREQ_1MHZ);     /* 1 MHz */
            tfpIoctl (SELFREQUENCYOUT, FREQ_1MHZ);     /* 1 MHz */
            break;
        case 2:
	  /*            tfpIoctl (pTfpDev, SELFREQUENCYOUT, FREQ_5MHZ);     /* 5 MHz */
            tfpIoctl (SELFREQUENCYOUT, FREQ_5MHZ);     /* 5 MHz */
            break;
        case 3:
	  /*            tfpIoctl (pTfpDev, SELFREQUENCYOUT, FREQ_10MHZ);    /* 10 MHz */
            tfpIoctl (SELFREQUENCYOUT, FREQ_10MHZ);    /* 10 MHz */
            break;
        default:
            printf ("Error Setting output frequency!");
            break;
    }
}

/**********************************************************************/
/* Description : Set Time Format */
/**********************************************************************/
void
pci_time_format ()
{
    int     time_format = 0;

    printf ("Select Time Format\n  0. Decimal\n  1. Binary\n\nSelect: ");
    scanf ("%d", &time_format);

    switch (time_format)
    {
        case 0:
	  /*            tfpIoctl (pTfpDev, SELTIMEFORMAT, TIME_DECIMAL); /* Decimal time */
            tfpIoctl (SELTIMEFORMAT, TIME_DECIMAL); /* Decimal time */
            break;
        case 1:
	  /*            tfpIoctl (pTfpDev, SELTIMEFORMAT, TIME_BINARY);	/* Binary time */
            tfpIoctl (SELTIMEFORMAT, TIME_BINARY);	/* Binary time */
            break;
        default:
            printf ("\nError Setting Time Format!");
            break;
    }
}

/**********************************************************************/
/* Description : Set Heartbeat Counters */
/**********************************************************************/
void
pci_heartbeat ()
{
    struct periodic sper;
    int     hrt_mode;
    int     counter1 = 10;
    int     counter2 = 10;

    sper.sync1pps = PERIODIC_SYNC;

    printf ("\n\n\n\t    Heartbeat Counters and Mode:\n\n");
    printf ("\t1. Synchronous mode\n");
    printf ("\t2. Asynchronous mode\n\n");
    printf ("\t   Select: ");
    scanf ("%i", &hrt_mode);
    printf ("\n\tEnter 4 hex value for counter1: \n");
    scanf ("%x", &counter1);
    printf ("\n\tEnter 4 hex value for counter2: \n");
    scanf ("%x", &counter2);

    if (hrt_mode == 2)
        sper.sync1pps = PERIODIC_NOSYNC;
    if (counter1 >= 2 && counter1 <= 65535)
        sper.n1 = counter1;
    if (counter2 >= 2 && counter2 <= 65535)
        sper.n2 = counter2;

    /*    tfpIoctl (pTfpDev, SETPERIODIC, (int) &sper); */
    tfpIoctl (SETPERIODIC, (int) &sper);
}


/**********************************************************************/
/* Description : Set Time */
/**********************************************************************/
void
pci_set_time ()
{
    int     choice;
    int     year = 1999;
    TFP_TM  tm;
    TFP_TV  tv;

    printf ("\n\n\n\t Set Time Functions :\n\n");
    printf ("\t1. Set Year\n");
    printf ("\t2. Set Time TM\n");
    printf ("\t3. Set Time TV\n");
    printf ("\t   Select: ");

    scanf ("%d", &choice);

    switch (choice)
    {
       case 1:
           printf ("\n\tEnter Year (1990 to 2050): \n");
           scanf ("%d", &year);
           if (year >= 1990 && year <= 2050)
	     /*              tfpIoctl (pTfpDev, SETYEAR, year);      /* Set year */
              tfpIoctl (SETYEAR, year);      /* Set year */
           else
              printf ("\nError: Value Out of Range");
           break;
       case 2:
           printf ("\n\tEnter Year : ");
           scanf ("%d", &(tm.tm.tm_year));
           printf ("\n\tEnter Year Day: ");
           scanf ("%d", &tm.tm.tm_yday);
           printf ("\n\tEnter Hour: ");
           scanf ("%d", &tm.tm.tm_hour);
           printf ("\n\tEnter Min: ");
           scanf ("%d", &tm.tm.tm_min);
           printf ("\n\tEnter Sec: ");
           scanf ("%d", &tm.tm.tm_sec);

	   /*           tfpIoctl (pTfpDev, SETTIMETM, (int)&tm); */
           tfpIoctl (SETTIMETM, (int)&tm);

           break;

       case 3:
           printf ("\n\tEnter Seconds: ");
           scanf ("%ld",&tv.tv.tv_sec);

	   /*           tfpIoctl (pTfpDev, SETTIMETV, (int)&tv); */
           tfpIoctl (SETTIMETV, (int)&tv);

           break;

       default:
           break;
    }

}


/**********************************************************************/
/* Description : Request Current Settings */
/**********************************************************************/
void
pci_req_current_settings ()
{
    const char smode[7][10] = {"Time Code", "Free Run", "Ext 1PPS", "RTC",
							   "Reserved", "Reserved", "GPS"};
    const char stm_fmt[2][10] = {"Decimal", "Binary"};
    int     tm_fmt = 0;
    int     year = 0;
    int     dac_val = 0;
    int     timing_off = 0;
    INT16   gen_off = 0;
    INT16   loc_off = 0;
    unsigned char  tc_fmt = ' ';
    unsigned char  tc_mod = ' ';
    unsigned char  clk_scr = ' ';
    UINT8   mode = 0;

    get.arg = GETDATA_MODE;     /* Timing Mode */
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        mode = (int) get.data.tmode;
    printf ("\nMode\t\t: %s", smode[mode]);

    get.arg = GETDATA_TFORMAT;  /* Time Format */
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        tm_fmt = (int) get.data.tformat;
    printf ("\nTime Format\t: %s", stm_fmt[tm_fmt]);

    get.arg = GETDATA_YEAR;     /* Year */
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        year = (int) get.data.year;
    printf ("\nYear\t\t: %d", year);

    get.arg = GETDATA_PER;
    /*    tfpIoctl (pTfpDev, GETDATA, (int)&get); */
    tfpIoctl (GETDATA, (int)&get);
    printf ("\nPeriodic Output\t: sync1pps=%x  n1=%x, n2=%x",
       get.data.period.sync1pps, get.data.period.n1,
       get.data.period.n2);

    get.arg = GETDATA_TCFMT;    /* Time Code Format */
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        tc_fmt = (unsigned char) get.data.tcformat;
    printf ("\nTime Code\t: IRIG %c", tc_fmt);

    get.arg = GETDATA_TCMOD;    /* Time Code Modulation */
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        tc_mod = (unsigned char) get.data.tcmod;
    printf ("\nCode Modulation\t: %c", tc_mod);

    get.arg = GETDATA_TOFF;
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        timing_off = (int) get.data.offset;
    printf ("\nTiming Offset\t: %d", timing_off);

    get.arg = GETDATA_UTCINFO;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nUTC\t : enable=%d,leapsec=%d,leapevt=%d,evttime=%x",
			get.data.utc.enable, get.data.utc.leapsec,
			get.data.utc.leapevt, get.data.utc.evttime);

    get.arg = GETDATA_TCOUTFMT;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nTCOUTFMT\t: %d", get.data.tcoutfmt);

    get.arg = GETDATA_LOCTMOFF;
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
         loc_off = get.data.locoff.offset;
    printf ("\nLoc Time Offset\t: %d", loc_off);

    get.arg = GETDATA_TCGENOFF;
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
         gen_off = get.data.genoff.offset;
    printf ("\nGen Time Offset\t: %d", gen_off);

    get.arg = GETDATA_LEAPSEC;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nLEAPSEC: ls_flag=%d, leap_tm=%d",
       get.data.leapsec.ls_flag, get.data.leapsec.leap_tm);

    get.arg = GETDATA_FWVER;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nFWVER\t: vmajor=%d, vminor=%d, month=%d, day=%d, year=%d",
       get.data.version.vmajor, get.data.version.vminor,
       get.data.version.month, get.data.version.day,
       get.data.version.year);


    get.arg = GETDATA_CLKSRC;
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        clk_scr = (unsigned char) get.data.clksrc;
    if (clk_scr == 'I')
        printf ("\nOscillator  \t: Internal");
    else
        printf ("\nOscillator  \t: External");


    get.arg = GETDATA_DAVAL;
    /*    if (!((tfpIoctl (pTfpDev, GETDATA, (int) &get)) < 0)) */
    if (!((tfpIoctl (GETDATA, (int) &get)) < 0))
        dac_val = (int) get.data.davalue;
    printf ("\nDAC Value\t: %x", dac_val);

    get.arg = GETDATA_JAMSC;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nJAMSC\t\t: %d", get.data.jsctl);

    get.arg = GETDATA_OSCCTL;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nOCSCTL\t: discctl=%d, phasectl=%d, phasestep=%d, gain=%x, con=%x",
       get.data.ocsctl.discctl, get.data.ocsctl.phasectl,
       get.data.ocsctl.phasestep, get.data.ocsctl.gain,
       get.data.ocsctl.con);

    get.arg = GETDATA_BATTSTAT;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nBATSTATT\t: %x", get.data.battstat);

    get.arg= GETDATA_CLKVAL;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nCLKVAL\t\t: %x", get.data.clkval);

    get.arg = GETDATA_DTFW;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nDTFW\t\t: %c%c%c%c%c%c%c%c%c%c%c",
      get.data.dtfw.dt01,
      get.data.dtfw.dt02,
      get.data.dtfw.dt03,
      get.data.dtfw.dt04,
      get.data.dtfw.dt05,
      get.data.dtfw.dt06,
      get.data.dtfw.dt07,
      get.data.dtfw.dt08,
      get.data.dtfw.dt09,
      get.data.dtfw.dt10,
      get.data.dtfw.dt11);

    get.arg = GETDATA_ASSEMB;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nASSEMB\t\t: %x", get.data.assemb);

    get.arg = GETDATA_TFPMODEL;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nTFPMODEL\t: %c%c%c%c%c%c%c%c",
      get.data.tfpm.tfp1,
      get.data.tfpm.tfp2,
      get.data.tfpm.tfp3,
      get.data.tfpm.tfp4,
      get.data.tfpm.tfp5,
      get.data.tfpm.tfp6,
      get.data.tfpm.tfp7,
      get.data.tfpm.tfp8);

    get.arg = GETDATA_SERIAL;
    /*    tfpIoctl (pTfpDev, GETDATA, (int) &get); */
    tfpIoctl (GETDATA, (int) &get);
    printf ("\nSERIAL Number\t: %x", get.data.serialnum);




    return;

}

/**********************************************************************
* tmPrint - print 'tm' struct
*
* RETURNS: OK
*/

LOCAL STATUS tmPrint
        (
                 TFP_TM * pTm
)

{
    (void) printf ("  tm_sec: %d\n", pTm->tm.tm_sec);
    (void) printf ("  tm_min: %d\n", pTm->tm.tm_min);
    (void) printf (" tm_hour: %d\n", pTm->tm.tm_hour);
    (void) printf (" tm_yday: %d\n", pTm->tm.tm_yday);
    (void) printf (" tm_year: %d\n", pTm->tm.tm_year);
    (void) printf ("    usec: %d\n", pTm->usec);
    (void) printf ("   hnsec: %d\n", pTm->hnsec);
    (void) printf ("  status: x%01X\n", pTm->status);

    return (OK);
}

/**********************************************************************
* tvPrint - print 'tv' struct
*
* RETURNS: OK
*/
static STATUS tvPrint
        (
                 TFP_TV * pTv
)

{
    (void) printf ("  tv_sec: %ld\n", pTv->tv.tv_sec);
    (void) printf ("    usec: x%ld\n", pTv->tv.tv_usec);
    (void) printf ("   hnsec: x%x\n", pTv->hnsec);
    (void) printf ("  status: x%01X\n", pTv->status);

    return (OK);
}

/**********************************************************************/
/* Description : read_time_menu */
/**********************************************************************/

void
read_time_menu ()
{
    int     choice;
    int     ioctl_choice;
    int     arg_choice;
    int     print_choice;
    TFP_TM  tfpTM;
    TFP_TV  tfpTV;


    printf ("\n\n\n  Read Time Formats:\n\n");
    printf ("\t1. RDTIMETM (Unlatched, DEC)\n");
    printf ("\t2. RDTIMERTMEQ (Latched, DEC)\n");
    printf ("\t3. RDTIMETV (Unlatched, BINARY)\n");
    printf ("\t4. RDTIMETVREQ (Latched, BINARY)\n\n");
    printf ("\t5. RDEVENTTM (Unlatched, DEC)\n");
    printf ("\t6. RDEVENTRTMEQ (Latched, DEC)\n");
    printf ("\t7. RDEVENTTV (Unlatched, BINARY)\n");
    printf ("\t8. RDEVENTTVREQ (Latched, BINARY)\n\n");
    printf ("\t9. TIMEREQUEST (Latch Time)\n");
    printf ("\t10. EVENTREQUEST (Latch Event)\n");
    printf ("\n Enter Choice : ");
    scanf ("%i", &choice);

    switch (choice)
    {
        case 1:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDTIMETM;
            arg_choice = (int) &tfpTM;
            break;
        case 2:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDTIMETMREQ;
            arg_choice = (int) &tfpTM;
            break;
        case 3:
            print_choice = TIME_BINARY;
            ioctl_choice = RDTIMETV;
            arg_choice = (int) &tfpTV;
            break;
        case 4:
            print_choice = TIME_BINARY;
            ioctl_choice = RDTIMETVREQ;
            arg_choice = (int) &tfpTV;
            break;
        case 5:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDEVENTTM;
            arg_choice = (int) &tfpTM;
            break;
        case 6:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDEVENTTMREQ;
            arg_choice = (int) &tfpTM;
            break;
        case 7:
            print_choice = TIME_BINARY;
            ioctl_choice = RDEVENTTV;
            arg_choice = (int) &tfpTV;
            break;
        case 8:
            print_choice = TIME_BINARY;
            ioctl_choice = RDEVENTTVREQ;
            arg_choice = (int) &tfpTV;
            break;
        case 9:
            print_choice = TIME_BINARY + TIME_DECIMAL;  /* none */
            ioctl_choice = TIMEREQUEST;
            arg_choice = (int) 0;
            break;
        case 10:
            print_choice = TIME_BINARY + TIME_DECIMAL;  /* none */
            ioctl_choice = EVENTREQUEST;
            arg_choice = (int) 0;
            break;

        default:
            printf ("Not implemented\n");
            return;
    }

    /*    if (tfpIoctl (pTfpDev, ioctl_choice, arg_choice) == ERROR) */
    if (tfpIoctl (ioctl_choice, arg_choice) == ERROR)
    {
        (void) printf ("tfpIoctl (%d) failed: ", ioctl_choice);
/*          printErrno (errnoGet ()); */
        return;
    }

    if (print_choice == TIME_DECIMAL)
        (void) tmPrint (&tfpTM);
    else if (print_choice == TIME_BINARY)
        (void) tvPrint (&tfpTV);

}

/**********************************************************************/
/* Description : pci_strobe_routines */
/**********************************************************************/
void pci_strobe_routines ()
{
}

/**********************************************************************/
/* Description : pci_interrupt_routines */
/**********************************************************************/
void pci_interrupt_routines ()
{
    int choice;
    unsigned int timeout;

    printf ("\n\n");
    printf ("Enter timeout (100 = 1 sec) :");
    scanf ("%i", &timeout);

    printf ("1. Wait on Event\n");
    printf ("2. Wait on Periodic\n");
    printf ("3. Wait on Strobe\n");
    printf ("4. Wait on 1 PPS\n");
    printf ("5. Wait on GPS\n");
    printf ("   Select : ");
    scanf ("%i", &choice);

    switch (choice)
    {
    case 1:
      /*      if (tfpIoctl (pTfpDev, BC635_INT_WAITONEVENT, timeout) == OK) */
      if (tfpIoctl (BC635_INT_WAITONEVENT, timeout) == OK)
        printf ("Got EVENT\n");
      else
        printf("EVENT Timeout\n");
      break;

    case 2:
      {
        int i;
        for (i=0; i < 5; i++)
        {
	  /*          if (tfpIoctl (pTfpDev, BC635_INT_WAITONPERIODIC, timeout) == OK) */
          if (tfpIoctl (BC635_INT_WAITONPERIODIC, timeout) == OK)
            printf ("Got PERIODIC\n");
          else
            printf("EVENT Timeout\n");
        }
      }
      break;

    case 3:
      /*      if (tfpIoctl (pTfpDev, BC635_INT_WAITONSTROBE, timeout) == OK) */
      if (tfpIoctl (BC635_INT_WAITONSTROBE, timeout) == OK)
        printf ("Got STROBE\n");
      else
        printf("EVENT Timeout\n");
      break;

    case 4:
      {
        int i;
        for (i=0; i < 5; i++)
        {
	  /*          if (tfpIoctl (pTfpDev, BC635_INT_WAITON1PPS, timeout) == OK) */
          if (tfpIoctl (BC635_INT_WAITON1PPS, timeout) == OK)
            printf ("Got 1PPS\n");
          else
            printf("EVENT Timeout\n");
        }
      }
        break;
    case 5:
      /*      if (tfpIoctl (pTfpDev, BC635_INT_WAITONGPS, timeout) == OK) */
      if (tfpIoctl (BC635_INT_WAITONGPS, timeout) == OK)
        printf ("Got GPS\n");
      else
        printf("EVENT Timeout\n");
      break;

    default:
        break;
    }

}
/***********************************************************************/
/* Description : pci_intstat_routines() */
/***********************************************************************/
void pci_intstat_routines ()
{
     int choice;
     UINT32 intstat;

     printf ("\n\n");
     printf ("1. Read Intstat \n");
     printf ("2. Set Intstat \n");
     printf ("3. Read Intmask \n");
     printf ("4. Set Intmask \n");
     printf ("   Select : ");
     scanf ("%i", &choice);

     switch (choice)
     {
       case 1:
          intstat = 0;
	  /*          tfpIoctl (pTfpDev, RDINTSTAT, (int)&intstat); */
          tfpIoctl (RDINTSTAT, (int)&intstat);
          printf ("INTSTAT = %x\n", intstat & INT_BITMASK);
          break;
       case 2:
          printf ("\nEnter value (in hex) : ");
          scanf ("%x", &intstat);
	  /*          tfpIoctl (pTfpDev, SETINTSTAT, intstat); */
          tfpIoctl (SETINTSTAT, intstat);
          break;
       case 3:
          intstat = 0;
	  /*          tfpIoctl (pTfpDev, RDINTMASK, (int)&intstat); */
          tfpIoctl (RDINTMASK, (int)&intstat);
          printf ("INTMASK = %x\n", intstat & INT_BITMASK);
          break;
       case 4:
          printf ("\nEnter value (in hex) : ");
          scanf ("%x", &intstat);
	  /*          tfpIoctl (pTfpDev, SETINTMASK, intstat); */
          tfpIoctl (SETINTMASK, intstat);
          break;

       default:
          break;
      }
}
/**********************************************************************/
/* Description : Main Menu */
/**********************************************************************/
void pci_sw_reset ()
{
  /*   tfpIoctl (pTfpDev, SOFTWARERESET, 0); */
   tfpIoctl (SOFTWARERESET, 0);
}

void pci_set_offsets ()
{
   UINT32 offset;
   UINT32 halfhour;
   int    choice;
   printf ("\n\n 1. Set Generator time offset \n");
   printf ("2. Set Local Time Offset \n");
   printf ("   Select : ");
   scanf ("%i", &choice);

   printf ("Enter Offset : ");
   scanf ("%i", &offset);
   printf ("Enter Half Hour (0 or 1) : ");
   scanf ("%i", &halfhour);

   switch (choice)
   {
      case 1:
       {
         struct tcgenoffset genoffset;

         genoffset.offset = (INT16) offset;
         genoffset.half_hour = (UINT8) halfhour;

	 /*         tfpIoctl (pTfpDev, SETGENTMOFFSET, (int)&genoffset); */
         tfpIoctl (SETGENTMOFFSET, (int)&genoffset);

       }
         break;
      case 2:
      {
        struct loctmoffset locoffset;

        locoffset.offset = (INT16) offset;
        locoffset.half_hour = (UINT8) halfhour;

	/*        tfpIoctl (pTfpDev, SETLOCTMOFFSET, (int) &locoffset); */
        tfpIoctl (SETLOCTMOFFSET, (int) &locoffset);

      }
         break;
      default:
         break;
   }
}

void pci_set_prop_delay()
{
  UINT32 delay;

  printf("Enter delay : ");
  scanf ("%i", &delay);

  /*  tfpIoctl (pTfpDev, SETPROPDELAY, delay); */
  tfpIoctl (SETPROPDELAY, delay);
}

void pci_set_leapsec_event ()
{
   INT32 tmp;
   UINT32 utmp;
   struct leapseconds leapsec;

   printf ("Enter Leap Sec Flag : ");
   scanf ("%i", &tmp);
   leapsec.ls_flag = (INT8) tmp;
   printf ("Enter Leap Time (Unix Format) : ");
   scanf ("%i", &utmp);
   leapsec.leap_tm = tmp;

   /*   tfpIoctl (pTfpDev, SETLEAPSECEVENT, (int) &leapsec); */
   tfpIoctl (SETLEAPSECEVENT, (int) &leapsec);


}

void pci_gps_example ()
{
    TFP_GPS_PACKET gps_packet;
    UINT32 id;
    int i;

    printf("Enter Packet to receive (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.packet_id = id;
    gps_packet.packet_length = 0;

    /*    tfpIoctl (pTfpDev, REQGPSPACKET, (int)&gps_packet); */
    tfpIoctl (REQGPSPACKET, (int)&gps_packet);

    if (gps_packet.packet_length == 0)
       printf("GPS Example - Could not receive GPS System Message\n");

    else
    {
       printf("Packet id = %x, length=%d\n", gps_packet.packet_id,
			  gps_packet.packet_length);
       for (i=0; i < gps_packet.packet_length-1; i++)
           printf ("[%x] = %x\n", i, gps_packet.packet_data[i]);
    }
}

void pci_gps_example2 ()
{
    TFP_GPS_PACKET gps_packet;
    UINT32 id;
    int i,tmp;

    printf("Enter Packet to Send (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.packet_id = id;
    printf("Enter Packet Length (decimal plus one) : ");
    scanf ("%d", &tmp);
    gps_packet.packet_length = (UINT8) tmp;
    for (i=0; i<gps_packet.packet_length-1;i++)
    {
       printf("Enter data (in hex) [%d] : ", i);
       scanf ("%x", &tmp);
       gps_packet.packet_data[i] = (UINT8) tmp;
    }

    /*    tfpIoctl (pTfpDev, SENDGPSPACKET, (int)&gps_packet); */
    tfpIoctl (SENDGPSPACKET, (int)&gps_packet);

}

void pci_gps_example3 ()
{
    TFP_GPS_SENDRECEIVE_PACKET gps_packet;
    UINT32 id;
    int i,tmp;

    printf("Enter Packet to Send (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.send.packet_id = id;
    printf("Enter Packet Length (decimal plus one) : ");
    scanf ("%d", &tmp);
    gps_packet.send.packet_length = (UINT8) tmp;
    for (i=0; i<gps_packet.send.packet_length-1;i++)
    {
       printf("Enter data (in hex) [%d] : ", i);
       scanf ("%x", &tmp);
       gps_packet.send.packet_data[i] = (UINT8) tmp;
    }

    printf("Enter Packet to receive (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.receive.packet_id = id;
    gps_packet.receive.packet_length = 0;



    /*    tfpIoctl (pTfpDev, MANREQGPSPACKET, (int)&gps_packet); */
    tfpIoctl (MANREQGPSPACKET, (int)&gps_packet);

    if (gps_packet.receive.packet_length == 0)
       printf("GPS Example - Could not receive GPS System Message\n");

    else
    {
       printf("Packet id = %x, length=%d\n", gps_packet.receive.packet_id,
			  gps_packet.receive.packet_length);
       for (i=0; i < gps_packet.receive.packet_length-1; i++)
           printf ("[%x] = %x\n", i, gps_packet.receive.packet_data[i]);
    }



}


void pci_set_gps_mode ()
{

  int mode;

  printf ("\n\t 1) Disable");
  printf ("\n\t 2) Enable");
  printf ("\n\t    Select : ");
  scanf ("%i", &mode);

  /*  tfpIoctl (pTfpDev, SETGPSMDFLG, mode); */
  tfpIoctl (SETGPSMDFLG, mode);
}


void pci_set_gps_time_format()
{

}


void pci_set_event()
{
  int i;
  unsigned int mode;

  printf ("\n\t 1) Disable Lockout");
  printf ("\n\t 2) Enable Lockout");
  printf ("\n\t 3) Event Input");
  printf ("\n\t 4) Programmable Periodic Input");
  printf ("\n\t 5) Falling Edge Active");
  printf ("\n\t 6) Rising Edge Active");
  printf ("\n\t 7) Capture Event Disable");
  printf ("\n\t 8) Capture Event Enable");
  printf ("\n\t 9) Enter Control Reg Value");
  printf ("\n\t    Select : ");
  scanf ("%i", &i);

  switch (i)
    {
    case 1:
      mode = CAPLOCK_DISABLE;
      break;
    case 2:
      mode = CAPLOCK_ENABLE;
      break;
    case 3:
      mode = EVCAP_EVENT;
      break;
    case 4:
      mode = EVCAP_PERIODIC;
      break;
    case 5:
      mode = EVENT_FALLING;
      break;
    case 6:
      mode = EVENT_RISING;
      break;
    case 7:
      mode = EVCAP_DISABLE;
      break;
    case 8:
      mode = EVCAP_ENABLE;
      break;
	case 9:
	  printf ("\n\t Enter Value (in hex) : ");
	  scanf("%x", &mode);
	  mode = mode | 0x10000000;				/* Mark this as a bit value */
	  break;
    default:
      return;
    }

  /*  tfpIoctl (pTfpDev, CONTROLEVENT, mode); */
  tfpIoctl (CONTROLEVENT, mode);
}

void pci_set_freq_source()
{
  int i;
  unsigned int mode;

  printf ("\n\t 1) Clock Internal");
  printf ("\n\t 2) Clock External");
  printf ("\n\t 3) Jamsync Enable");
  printf ("\n\t 4) Jamsycn Disable");
  printf ("\n\t 5) Discipline Enable");
  printf ("\n\t 6) Discipline Disable");
  printf ("\n\t 7) Force Jamsync");
  printf ("\n\t    Select : ");
  scanf ("%i", &i);

  switch (i)
    {
    case 1:
      mode = CLOCK_INTERNAL;
      break;
    case 2:
      mode = CLOCK_EXTERNAL;
      break;
    case 3:
      mode = JAMSYNC_ENABLE;
      break;
    case 4:
      mode = JAMSYNC_DISABLE;
      break;
    case 5:
      mode = DISCIPLINE_ENABLE;
      break;
    case 6:
      mode = DISCIPLINE_DISABLE;
      break;
    case 7:
      mode = FORCE_JAMSYNC;
      break;
    default:
      return;
    }

  /*  tfpIoctl (pTfpDev, CONTROLEVENT, mode); */
  tfpIoctl (CONTROLEVENT, mode);
}


/***********************************************************************/
/*        root: Sets up the evaluation program execution.              */
/*                                                                     */
/*        NOTE: Executes as task 'ROOT'.                               */
/***********************************************************************/
void root(void)
{
  unsigned long date, time, ticks;
  int     menu;


/*---------------------------------------------------------------------*/
/* Set date to May 1, 1995, time to 8:30 AM, and start the system      */
/* clock running.                                                      */
/*---------------------------------------------------------------------*/
  date = (1995 << 16) + (5 << 8) + 1;
  time = (8 << 16) + (30 << 8);
  ticks = 0;
  tm_set(date, time, ticks);

#if !SC_AUTOINIT
  de_init(DEV_TIMER, iopb, &ioretval, &data_ptr);
  de_init(CONSOLE, iopb, &ioretval, &data_ptr);
#endif

  PciConfig();

  /* tfpInit(&pTfpDev); */
  tfpInit();


  do
    {
      printf ("\n  Datum Inc. Bancomm-Timing Division\n\n");
      printf ("    bc635PCI Time & Frequency Processor\n\n");
      printf ("1.  Read time/Event Functions    14. Set Offsets \n");
      printf ("2.  Select Operational Mode      15. Prop Delay \n");
      printf ("3.  Select Time Code Format      16. Leap Sec \n");
      printf ("4.  Select Output Freq           17. Capunlock \n");
      printf ("5.  Select Time Format           18. GPS Example1 \n");
      printf ("6.  Program Heartbeat            19. Set GPS Mode \n");
      printf ("7.  Set Time Functions           20. Set GPS TM Fmt\n");
      printf ("8.  Select Clock Source          21. GPS Example2\n");
      printf ("9.  Current Settings             22. GPS Example3\n");
      printf ("10. Strobe / Strobetv            23. Set Debugging ON\n");
      printf ("11. Interrupt Routines           24. Set Debugging OFF\n");
      printf ("12. Interrupt Stat Routines      25. Set Event mode\n");
      printf ("13. Lucky software reset         26. Set Freq Int'rl/Ext'rl\n");
      printf ("0.  Exit the Program \n\n");
      printf ("    Select: ");
      scanf ("%i", &menu);

      switch (menu)
        {
        case 1:
          read_time_menu ();
          break;
        case 2:
          pci_mode ();
          break;
        case 3:
          pci_time_code ();
          break;
        case 4:
          pci_out_freq ();
          break;
        case 5:
          pci_time_format ();
          break;
        case 6:
          pci_heartbeat ();
          break;
        case 7:
          pci_set_time ();
          break;
        case 8:                             /* pci_set_clock(); */
          break;
        case 9:
          pci_req_current_settings ();
          break;
        case 10:
          pci_strobe_routines ();
          break;
        case 11:
          pci_interrupt_routines ();
          break;
        case 12:
          pci_intstat_routines ();
          break;
        case 13:
          pci_sw_reset ();
          break;
        case 14:
          pci_set_offsets ();
          break;
        case 15:
          pci_set_prop_delay();
          break;
        case 16:
          pci_set_leapsec_event();
          break;
        case 17:
          /* tfpIoctl (pTfpDev, CAPUNLOCK, 0); */
          tfpIoctl (CAPUNLOCK, 0);
          break;
        case 18:
          pci_gps_example();
          break;
        case 19:
          pci_set_gps_mode();
          break;
        case 20:
          pci_set_gps_time_format();
          break;
        case 21:
          pci_gps_example2();
          break;
        case 22:
          pci_gps_example3();
          break;
        case 23:
          bc635Debug = 1;
          break;
        case 24:
          bc635Debug = 0;
          break;
        case 25:
          pci_set_event();
          break;
        case 26:
          pci_set_freq_source();
          break;
        case 0:
        default:
/*            return OK; */
          exit(0);
        }
    } while (1);
}

