/* These are the VxWorks to pSOS+ wrapper functions for the PCI bus */

#include "sys_conf.h"
#include "psos.h"
#include "bsp.h"
#include <bspfuncs.h>
#include "powerpc/isrppc.h"
#include "../devices/pci/pcihdr.h"
#include <stdio.h>
#include "generalcore.h"
#include "integer_type.h"
#include "vxWorks_wrapper.h"


STATUS pciFindDevice (UINT16 vendorID, UINT16 deviceID, int index,
					  int *pciBridgeNo, int *pciBusNo, int *pciDevNo,
					  int *pciFuncNo, void **pciLoc)
{
	UINT32  pciID;
	PCI_LOC	*p_loc;

	pciID = ((UINT32)deviceID << 16) | vendorID;

/*		pciLoc = PciFindAndClaim(&pciID, 1, PCI_NO_AUTO_UNCLAIM, 0, 0); */

	if ((p_loc = (PCI_LOC *)PciFindDev(&pciID, 1, (UINT32)index,
									   PCI_ANY_HOST_BRIDGE)) ==
		(void *)NULL) /* Board ID, IDlen, number to skip, host bridge */
	{
	    printf("BC635/7(TFP): device not installed\n");
		return(ERROR);
	}

	*pciLoc = p_loc;

	*pciBridgeNo = p_loc->hostBridge;
	*pciBusNo = p_loc->bus;
	*pciDevNo = p_loc->device;
	*pciFuncNo = p_loc->function;

	return(OK);
}


/* NOTE: bridgeNo is an extra parameter not contained in the VxWorks function */
STATUS pciConfigInLong (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT32 *value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	*value = (UINT32)PciCfgRead(&pci_loc, baseAddress, PCI_REG_32BIT);

	return (OK);
}


STATUS pciConfigInWord (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT16 *value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	*value = (UINT32)PciCfgRead(&pci_loc, baseAddress, PCI_REG_16BIT);

	return (OK);
}


STATUS pciConfigInByte (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT8 *value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	*value = (UINT32)PciCfgRead(&pci_loc, baseAddress, PCI_REG_8BIT);

	return (OK);
}


STATUS pciConfigOutLong (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT32 value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	PciCfgWrite(&pci_loc, baseAddress, value, PCI_REG_32BIT);

	return (OK);
}


STATUS pciConfigOutWord (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT16 value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	PciCfgWrite(&pci_loc, baseAddress, value, PCI_REG_16BIT);

	return (OK);
}


STATUS pciConfigOutByte (int bridgeNo, int pciBusNo, int pciDevNo,
						int pciFuncNo, UINT32 baseAddress, UINT8 value)
{
    PCI_LOC pci_loc;

	pci_loc.hostBridge = bridgeNo;
	pci_loc.bus = pciBusNo;
	pci_loc.device = pciDevNo;
	pci_loc.function = pciFuncNo;

	PciCfgWrite(&pci_loc, baseAddress, value, PCI_REG_8BIT);

	return (OK);
}


void sysPciWrite32 (UINT32 address, UINT32 val)
{
    PciWrite32(address, val);
}


void sysPciRead32 (UINT32 address, UINT32 * pVal)
{
    *pVal = PciRead32(address);
}


UINT32 sysPciInLong (UINT32 address)
{
    return(PciRead32(address));
}


void sysPciOutLong (UINT32 address, UINT32 val)
{
    PciWrite32(address, val);
}


UINT16 sysPciInWord (UINT32 address)
{
    return(PciRead16(address));
}


void sysPciOutWord (UINT32 address, UINT16 val)
{
    PciWrite16(address, val);
}


UINT8 sysPciInByte (UINT32 address)
{
    return(PciRead8(address));
}


void sysPciOutByte (UINT32 address, UINT8 val)
{
    PciWrite8(address, val);
}


STATUS intContext ()
{
}


ULONG sysClkRateGet(void)
{
	return(100);
}


